/*
 * Created on 26/01/2007
 */

package fbtypes;

import org.jdom.Element;

public class Algorithm extends CommonInformation {

	private FBD fbd = null;
	private ST st = null;
	private LD ld = null;
	private Other other = null;
	// algorithmType: 1 - FBD, 2 - ST, 3 - LD, 4 - Other
	private int algorithmType = 0;

	public Algorithm(String Name, FBD fbd, String Comment) {
		super(Name, Comment);
		this.fbd = fbd;
		algorithmType = 1;
	}

	public Algorithm(String Name, ST st, String Comment) {
		super(Name, Comment);
		this.st = st;
		algorithmType = 2;
	}

	public Algorithm(String Name, LD ld, String Comment) {
		super(Name, Comment);
		this.ld = ld;
		algorithmType = 3;
	}

	public Algorithm(String Name, Other other, String Comment) {
		super(Name, Comment);
		this.other = other;
		algorithmType = 4;
	}

	public String toString() {
		return "Algorithm";
	}

	public Element toXML() {
		Element algorithmElement = new Element("Algorithm");

		algorithmElement.setAttribute("Name", Name);
		if(Comment != null)
			algorithmElement.setAttribute("Comment", Comment);

		switch(algorithmType) {
		case 1:
			algorithmElement.addContent(fbd.toXML());
			break;
		case 2:
			algorithmElement.addContent(st.toXML());
			break;
		case 3:
			algorithmElement.addContent(ld.toXML());
			break;
		case 4:
			algorithmElement.addContent(other.toXML());
			break;
		default:
			System.err.println("Unspecified algorithm type");
			System.exit(0);
			break;
		}
		return algorithmElement;
	}

	public void print() {
		System.out.println("Name: " + Name);
		System.out.println("Comment: " + Comment);

		switch(algorithmType) {
		case 1:
			System.out.println("FBD: ");
			fbd.print();
			break;
		case 2:
			System.out.println("ST: ");
			st.print();
			break;
		case 3:
			System.out.println("LD: ");
			ld.print();
			break;
		case 4:
			System.out.println("Other: ");
			other.print();
			break;
		default:
			System.err.println("Unspecified algorithm type");
			System.exit(0);
			break;
		}
	}

	public FBD getFbd() {
		return fbd;
	}

	public void setFbd(FBD fbd) {
		this.fbd = fbd;
	}

	public LD getLd() {
		return ld;
	}

	public void setLd(LD ld) {
		this.ld = ld;
	}

	public Other getOther() {
		return other;
	}

	public void setOther(Other other) {
		this.other = other;
	}

	public ST getSt() {
		return st;
	}

	public void setSt(ST st) {
		this.st = st;
	}
}
