/*
 * Created on 27/01/2007
 * Only Application and attributes are included
 * This is System element however due to the namespace conflict renamed to FBSystem
 */

package fbtypes;

import java.util.Hashtable;

import org.jdom.Element;

public class FBSystem extends CommonInformation {

	// Use name as Key for applications
	private Hashtable<String, Application> applications = null;
	// Use Integer from 0 as Key
	private Hashtable<Integer, VersionInfo> versionInfos = null;
	private Identification identification = null;
	
	private Hashtable<String, FBDevice> devices = null;
	

	public FBSystem(String Name, String Comment, Identification identification,
			Hashtable<Integer, VersionInfo> versionInfos,
			Hashtable<String, Application> applications,
			Hashtable<String, FBDevice> devices) {
		super(Name, Comment);
		this.applications = applications;
		this.versionInfos = versionInfos;
		this.identification = identification;
		this.devices = devices;
	}

	public String toString() {
		return "FBSystem";
	}

	public Element toXML() {
		Element fbSystemElement = new Element("System");
		fbSystemElement.setAttribute("Name", Name);
		if(Comment != null)
			fbSystemElement.setAttribute("Comment", Comment);

		if(identification != null) {
			fbSystemElement.addContent(identification.toXML());
		}

		for(VersionInfo versionInfo : versionInfos.values()) {
			fbSystemElement.addContent(versionInfo.toXML());
		}

		if(applications != null) {
			for(Application application : applications.values()) {
				fbSystemElement.addContent(application.toXML());
			}
		}
		return fbSystemElement;
	}

	public void print() {
		super.print();
		if(identification != null) {
			identification.print();
		}
		for(VersionInfo versionInfo : versionInfos.values()) {
			versionInfo.print();
		}
		if(applications != null) {
			for(Application application : applications.values()) {
				application.print();
			}
		}
		else
			System.out.println("applications is empty");
	}

	public Hashtable<String, Application> getApplications() {
		return applications;
	}

	public void setApplications(Hashtable<String, Application> applications) {
		this.applications = applications;
	}

	public Identification getIdentification() {
		return identification;
	}

	public void setIdentification(Identification identification) {
		this.identification = identification;
	}

	public Hashtable<Integer, VersionInfo> getVersionInfos() {
		return versionInfos;
	}

	public void setVersionInfos(Hashtable<Integer, VersionInfo> versionInfos) {
		this.versionInfos = versionInfos;
	}

	public Hashtable<String, FBDevice> getDevices() {
		return devices;
	}

}
