//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb;

import java.awt.*;
import java.io.*;
import javax.swing.*;

// Referenced classes of package fb.tools.swing.editor:
//			FBJDialogOKCancel

/**
 * Compiles the translated Java code that appears on Java tab
 * 
 * @author WD, AS
 * 
 * @version 20061231/WD, AS
 */
public class Compiler extends CompilerAction
	implements Runnable, FilenameFilter
{

	public Compiler(Frame parent)
	{
		super(parent, false);
		srcpane = new JTextArea("", 12, 72);
		srcScroll = new JScrollPane(srcpane);
		errpane = new JTextArea("", 12, 72);
		errScroll = new JScrollPane(errpane);
		panel.setLayout(new GridLayout(1, 2));
		srcpane.setFont(new Font("monospaced", 0, 12));
		srcpane.setToolTipText("This pane contains the Java source code with line numbers.");
		panel.add(srcScroll);
		panel.add(errScroll);
		errpane.setFont(srcpane.getFont());
		errpane.setToolTipText("This pane contains the compiler output and other messages.");
		okButton.setToolTipText("Closes the compiler window.");
		buttonPanel.remove(cancelButton);
		Dimension ssz = srcpane.getPreferredSize();
		Dimension esz = errpane.getPreferredSize();
		setSize(new Dimension(ssz.width, ssz.height + esz.height));
		setTitle("OOONEIDA FBench Compiler Build 20060826");
	}

	public static void log(InputStream strm, JTextArea pane, boolean lnos)
	{
		try
		{
			LineNumberReader lrdr = new LineNumberReader(new InputStreamReader(strm));
			for(String line = lrdr.readLine(); line != null; line = lrdr.readLine())
			{
				if(lnos)
					pane.append(lrdr.getLineNumber() + " ");
				pane.append(line + '\n');
			}

			lrdr.close();
		}
		catch(IOException ioexception) { }
	}

	public void compile(File f, String compilecmd)
	{
		status.setText("Compiling " + f.getName() + " now");
		try
		{
			log(new FileInputStream(f), srcpane, true);
		}
		catch(FileNotFoundException filenotfoundexception) { }
		try
		{
			Process compile = Runtime.getRuntime().exec(compilecmd + " \"" + f.getCanonicalPath() + "\"");
			errstrm = compile.getErrorStream();
			(new Thread(this)).start();
			log(compile.getInputStream(), srcpane, false);
			compile.waitFor();
		}
		catch(IOException e)
		{
			status.setText("Cannot find the Java compiler in: "+compilecmd);
			errpane.append("Cannot find the Java compiler in: \n   ");
			errpane.append(compilecmd);
			errpane.append("\nPlease Modify javac preference in the FBench/preferences.ini file.");
		}
		catch(InterruptedException e)
		{
			errpane.append(e.toString());
		}
		if(errpane.getText().length() == 0)
		{
			status.setText("Compiled " + f.getName() + " Successfully");
			errpane.setText("Compiled successfully! \nUse Launch in Run Menu to start the Function Blocks.");
		} else
		{
			status.setText("Compiled " + f.getName() + " Failed!");
		}
	}

	public void run()
	{
		log(errstrm, errpane, false);
	}

	public boolean accept(File dir, String name)
	{
		return name.endsWith(".java");
	}

	private JTextArea srcpane;
	private JScrollPane srcScroll;
	private JTextArea errpane;
	private JScrollPane errScroll;
	private InputStream errstrm;
}