//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.

//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License
package org.oooneida.wb;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.tree.TreeSelectionModel;

import org.oooneida.wb.dom.DOMTextModel;
import org.oooneida.wb.dom.DocumentSource;
import org.oooneida.wb.graph.GraphViewTransfer;
import org.oooneida.wb.plugin.PluginManager;
import org.oooneida.wb.tree.LibraryTreeNode;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

import fb.rt.tools.SystemLogger;
import fb.rt.tools.SystemManager;
import fb.tools.FBELibrarySpec;
import fb.tools.FileLibrary;
import fb.tools.swing.editor.FBJPreferences;

/**
 * A desktop for multiple Editors.
 * 
 * @authors JP,WD,JHC
 * 
 * @version 20070422/JP  - Separated plug-in features from the kernel parts
 * @version 20061127/WD  - Remove Left Tree,Add open function to the menu
 * @version 20060914/WD  - Added System Configurations Debugging
 * @version 20051129/JHC - Eliminated use of DTDs; ElementTableModel now sets up
 *          own attribute names.
 * @version 20051128/JHC - Set up main pane as JSplitPane with library view as
 *          left element.
 * @version 20050810/JHC - First OOONEIDA version.
 * @version 20050803/JHC - Changed desktop to JTabbedPane.
 * @version 20050709/JHC
 */
public class FBench extends JFrame implements DocumentSource, ActionListener, MenuListener {

	public static final String productID = "IEC 61499 FBench";

	public static final String version = "v0.5 Build 20070610a";

	public static Hashtable processes = new Hashtable();

	private Hashtable<String, Object> instances = new Hashtable<String, Object>();

	/** The TransferHandler for dragging components into GraphViews. */
	private GraphViewTransfer gvhandler;

	private Properties preferences = new Properties();

	private JTabbedPane desktop = new JTabbedPane();

	private JToolBar toolBar = new JToolBar(1);

	/** For View Menu*/
	private JFrame viewFrame = new JFrame("FBench Library");

	private JPopupMenu refresh_view = new JPopupMenu();

	private JMenuItem refresh_viewwini = createMenuItem("Refresh16","Refresh","reloadLibraryTree",'R');

	private JMenu fileMenu = new JMenu("File");

	private JMenu fileNewMenu = new JMenu("New");

	private JMenu fileNewFBMenu = new JMenu("FB Type");

	private JMenu fileNewDataMenu = new JMenu("Data Type");

	private JMenuItem newBFB = createMenuItem("New16", "Basic FB", "org.oooneida.wb.FBench.newBFB", 'B');

	private JMenuItem newCFB = createMenuItem("New16", "Composite FB", "org.oooneida.wb.FBench.newCFB", 'C');

	private JMenuItem newProxy = createMenuItem("New16", "Proxy FB", "org.oooneida.wb.FBench.newProxy", 'P');

	private JMenuItem newSI = createMenuItem("New16", "Service Interface FB", "org.oooneida.wb.FBench.newSI", 'S');

	private JMenuItem newSimple = createMenuItem("New16", "Simple FB", "org.oooneida.wb.FBench.newSimple", 'I');

	private JMenuItem newAdapt = createMenuItem("New16","Adapter Type","org.oooneida.wb.FBench.newAdapt",'A');

	private JMenuItem newRes = createMenuItem("New16","Resource Type","org.oooneida.wb.FBench.newRes",'N');

	private JMenuItem newDev = createMenuItem("New16","Device Type","org.oooneida.wb.FBench.newDev",'D');

	private JMenuItem newSys = createMenuItem("New16","System","org.oooneida.wb.FBench.newSys",'Y');

	private JMenuItem fileOpen = createMenuItem("Open16","Open...","org.oooneida.wb.FBench.FileOpen",'O');

	private JMenu fileSaveAsMenu = new JMenu("Save As");

	private JMenuItem fileSaveAsXMLItem = createMenuItem("xmlObject", "XML...",
			"org.oooneida.wb.FBench.fileSaveAsXML", 'X');

	private JMenuItem ProjectEnv = createMenuItem("New16","Project Environment","org.oooneida.wb.FBench.projectenv");	

	private JMenuItem fileExitItem = createMenuItem("Close", "Exit", "org.oooneida.wb.FBench.fileExit");

	private JMenu editMenu = new JMenu("Edit");

	private JMenuItem editCutItem = createMenuItem("Cut16", "Cut",
			"org.oooneida.wb.FBench.editAction", "Move selected text to clipboard");

	private JMenuItem editCopyItem = createMenuItem("Copy16", "Copy",
			"org.oooneida.wb.FBench.editAction", "Copy text to clipboard");

	private JMenuItem editPasteItem = createMenuItem("Paste16", "Paste",
			"org.oooneida.wb.FBench.editAction", "Replace selected text with clipboard contents");

	private JMenuItem editDeleteItem = createMenuItem("Delete16", "Delete",
			"org.oooneida.wb.FBench.editAction", "Permanently delete selected text");

	private JMenuItem editFindReplaceItem = createMenuItem("Find16",
			"Find/Replace...", "org.oooneida.wb.FBench.editAction", "Open Find/Replace Dialog");

	private JMenu viewMenu = new JMenu("View");

	private JCheckBoxMenuItem treeViewMenu = new JCheckBoxMenuItem("Display Library Tree Window",
			IconFactory.getIcon("view16"));

	private JMenu compileMenu = new JMenu("Compile");

	private JMenuItem compiletoJava = createMenuItem("compile16","Compile to Java",
			"org.oooneida.wb.FBench.fileSaveAsJava","Compile to Java with remote debugging(FBRT)",'J');

	private JMenu runMenu = new JMenu("Run");

	private JMenu runLaunchMenu = new JMenu("Launch");

	private JMenuItem runLaunchItem = createMenuItem("fb", "Launch",
			"org.oooneida.wb.FBench.runLaunch","Launches a system configuration or an instance of an FB type.");

	private JMenu helpMenu = new JMenu("Help");

	private JMenuItem helpItem = createMenuItem("Help16", "FBench Help",
	"org.oooneida.wb.FBench.help");

	private JMenuItem helpHowToItem = createMenuItem("Help16", "How To...",
	"org.oooneida.wb.FBench.help");

	private JMenuItem helpAboutItem = createMenuItem("Help16","About FBench", "org.oooneida.wb.FBench.help");

	public FBench() {
		super();
		Library.setDocumentSource(this);
		setupPreferences();
		setTitle(FBench.productID);
		setIconImage(IconFactory.getIcon("fbench24.png").getImage());
		setLayout(new BorderLayout());
		desktop.setFont(new Font("SansSerif", Font.PLAIN, 10));

		instances.put("org.oooneida.wb.FBench", this);

		final FBench me = this;
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				me.windowClosing(evt);
			}
		});

		menuBarInit();

		toolBar.setFloatable(true);

		String[] newFunctions = {"Parse XML","Save Now","Close Tab","Previous Tab","Next Tab"};
		String[] newPics = {"org/oooneida/wb/images/Refresh16.gif","org/oooneida/wb/images/SaveAs16.gif","org/oooneida/wb/images/Close16.gif",
				"org/oooneida/wb/images/arrow15l.gif","org/oooneida/wb/images/arrow15r.gif"};
		buildToolButton(toolBar,newFunctions,newPics);
		getContentPane().add(toolBar,BorderLayout.WEST);

		getContentPane().add(desktop, BorderLayout.CENTER);
		setBounds(100, 100, 800, 600);
		Vector helpArg = new Vector();
		helpArg.add(new String("About"));
		if (help(helpArg) != JOptionPane.OK_OPTION)
			System.exit(-1);
		setVisible(true);

		String openfiles = preferences.getProperty("openfiles");
		if (openfiles != null) {
			StringTokenizer toker = new StringTokenizer(openfiles, ",");
			while (toker.hasMoreElements())
				open(toker.nextToken());
		}
	}

	public void menuSelected(MenuEvent evt) {
		Object src = evt.getSource();
		if (src == editMenu)
			editMenuSelected();
		else if (src == runMenu)
			runMenuSelected();
		else if (src == compileMenu)
			CompileSelected();
		else{
			String actionCommand = ((JMenu)evt.getSource()).getActionCommand();
			String callerName = actionCommand.substring(0, actionCommand.lastIndexOf("."));
			String methodName = actionCommand.substring(actionCommand.lastIndexOf(".")+1);
			
			Vector args = new Vector();
			args.add((JMenu) evt.getSource());
			args.add(this);
			
			try {
				Class caller = Class.forName(callerName);
			
			Method method = caller.getMethod(methodName, Vector.class);
			
			java.util.Hashtable<String,Object> instances = getInstances();
			
			Object instance = instances.get(callerName);
			if(instance == null){
				instance = caller.newInstance();
				instances.put(callerName, instance);
			}
			
			method.invoke(instance, args);
			
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
	}

	public void actionPerformed(ActionEvent evt) {
		String cmd = ((AbstractButton) evt.getSource()).getText();
		String actionCommand = evt.getActionCommand();
		String callerName = actionCommand.substring(0, actionCommand.lastIndexOf("."));
		String methodName = actionCommand.substring(actionCommand.lastIndexOf(".")+1);

		Class[] parameterTypes = new Class[1];
		parameterTypes[0] = Vector.class;

		Vector args = new Vector();
		args.add(cmd);
		args.add(this);
		String cmdtxt = methodName + "(\"" + cmd + "\")\n";
		try {
			Class caller = Class.forName(callerName);
			Method method = caller.getMethod(methodName, parameterTypes);

			Object instance = instances.get(callerName);
			if(instance == null){
				instance = caller.newInstance();
				instances.put(callerName, instance);
			}

			method.invoke(instance, args);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			System.err.println(cmdtxt + e);
		} catch (SecurityException e) {
			e.printStackTrace();
			System.err.println(cmdtxt + e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			System.err.println(cmdtxt + e);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			System.err.println(cmdtxt + e);
			System.err.println("Cause: " + e.getCause());
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			System.err.println(cmdtxt + e);
		}
		catch (Exception e){
			System.err.print(e);
		}
	}

	/**
	 *  
	 */
	private void menuBarInit() {
		JMenuBar mb = new JMenuBar();
		PluginManager plugin = new PluginManager(this);

		mb.add(fileMenu);
		fileMenu.add(fileNewMenu);
		fileNewMenu.add(fileNewFBMenu);
		fileNewFBMenu.add(newBFB);
		fileNewFBMenu.add(newCFB);
		fileNewFBMenu.add(newProxy);
		fileNewFBMenu.add(newSI);
		fileNewFBMenu.add(newSimple);
		fileNewMenu.add(fileNewDataMenu);
		fileNewMenu.add(newAdapt);
		fileNewMenu.add(newRes);
		fileNewMenu.add(newDev);
		fileNewMenu.add(newSys);
		fileNewMenu.setIcon(IconFactory.getIcon("New16"));
		fileMenu.add(fileOpen);
		fileMenu.addSeparator();
		fileMenu.add(fileSaveAsMenu);
		fileSaveAsMenu.add(this.fileSaveAsXMLItem);
		fileSaveAsMenu.setIcon(IconFactory.getIcon("SaveAs16"));
		fileMenu.addSeparator();
		fileMenu.add(ProjectEnv);
		fileMenu.addSeparator();
		fileMenu.add(fileExitItem);

		mb.add(editMenu);
		editMenu.addMenuListener(this);
		editCutItem.setAccelerator(KeyStroke.getKeyStroke("control X"));
		editMenu.add(editCutItem);
		editCopyItem.setAccelerator(KeyStroke.getKeyStroke("control C"));
		editMenu.add(editCopyItem);
		editPasteItem.setAccelerator(KeyStroke.getKeyStroke("control V"));
		editMenu.add(editPasteItem);
		editDeleteItem.setAccelerator(KeyStroke.getKeyStroke("DELETE"));
		editMenu.add(editDeleteItem);
		editMenu.addSeparator();
		editFindReplaceItem.setAccelerator(KeyStroke.getKeyStroke("control F"));
		editMenu.add(editFindReplaceItem);

		mb.add(viewMenu);
		viewMenu.add(treeViewMenu);
		treeViewMenu.addActionListener(this);
		treeViewMenu.setActionCommand("org.oooneida.wb.FBench.runViewMenu");

		refresh_viewwini.setIcon(new ImageIcon("org/oooneida/wb/images/Refresh16.gif"));
		refresh_view.add(refresh_viewwini);

		mb.add(compileMenu);
		compileMenu.add(compiletoJava);
		compileMenu.addMenuListener(this);

		mb.add(runMenu);
		runLaunchMenu.setIcon(IconFactory.getIcon("Play16"));
		runLaunchMenu.add(runLaunchItem);
		runMenu.add(runLaunchMenu);
		runMenu.addMenuListener(this);

		for(JMenuItem item : plugin.loadMenuItems()){
			mb.add(item);
		}
		
		mb.add(helpMenu);
		boolean canbrowse = (preferences.getProperty("browser") != null);
		helpMenu.add(helpItem);
		helpItem.setEnabled(canbrowse);
		helpMenu.add(helpHowToItem);
		helpHowToItem.setEnabled(canbrowse);
		helpMenu.addSeparator();
		helpMenu.add(helpAboutItem);

		setJMenuBar(mb);

	}

	private void buildToolButton(JToolBar toolBar,String[] functions,String[] pics){
		JButton button;

		for(int i=0; i<functions.length; i++){
			ActionSet action = new ActionSet(functions[i],new ImageIcon(pics[i]));
			button = toolBar.add(action);
			button.setActionCommand(functions[i]);
			button.setToolTipText((String)action.getValue(Action.NAME));
			button.setEnabled(false);
		}
	}

	private void CheckToolBar(){
		toolBar.getComponent(0).setEnabled(desktop.getComponentCount() > 0);
		toolBar.getComponent(1).setEnabled(desktop.getComponentCount() > 0);
		toolBar.getComponent(2).setEnabled(desktop.getComponentCount() > 0);
		toolBar.getComponent(3).setEnabled(desktop.getSelectedIndex() > 0);
		toolBar.getComponent(4).setEnabled(desktop.getSelectedIndex() >= 0 && desktop.getComponentCount() > 1);
	}

	protected void windowClosing(WindowEvent evt) {
		dispose();
		System.exit(0);
	}

	public void fileExit(String s) {
		dispose();
	}
	/**
	 * Answers an InputSource on a file with the given name and suffix in the
	 * <B>src </B> subdirectory tree; if none is found, shows an error dialog
	 * and answers <TT>null</TT>.
	 * 
	 * @param docname
	 *            The document name in the form <TT>filename.suffix</TT>.
	 */
	public InputSource getSource(String docname) {
		InputSource src = getSource(new File("src"), docname);
		if (src == null)
			JOptionPane.showMessageDialog(this, "Can't find source for "
					+ docname + '.', "File Open Failed",
					JOptionPane.WARNING_MESSAGE);
		return src;
	}

	/**
	 * Answers an InputSource on a file with the given name and suffix in the
	 * given subdirectory tree. If none is found, answers <TT>null</TT>.
	 * 
	 * @param dir
	 *            The root directory for the search.
	 * @param docname
	 *            The document name in the form <TT>filename.suffix</TT>.
	 */
	private InputSource getSource(File dir, String docname) {
		try {
			return new InputSource(new FileInputStream(new File(dir, docname)));
		} catch (FileNotFoundException e) {
			File[] dirs = dir.listFiles(new FileFilter() {
				public boolean accept(File fl) {
					return fl.isDirectory();
				}
			});
			for (int i = 0; i < dirs.length; i++) {
				InputSource src = getSource(dirs[i], docname);
				if (src != null)
					return src;
			}
		}
		return null;
	}

	public static void main(String[] args) {
		new FBench();
	}

	private JMenuItem createMenuItem(String label, String actionCommand) {
		JMenuItem item = new JMenuItem(label);
		item.addActionListener(this);
		item.setActionCommand(actionCommand);
		return item;
	}

	private JMenuItem createMenuItem(String iconName, String label,
			String actionCommand) {
		JMenuItem item = createMenuItem(label, actionCommand);
		item.setIcon(IconFactory.getIcon(iconName));
		return item;
	}

	private JMenuItem createMenuItem(String iconName, String label,
			String actionCommand, String toolTip) {
		JMenuItem item = createMenuItem(iconName, label, actionCommand);
		item.setToolTipText(toolTip);
		return item;
	}

	private JMenuItem createMenuItem(String iconName, String label,
			String actionCommand, String toolTip, char accelerator) {
		JMenuItem item = createMenuItem(iconName, label, actionCommand);
		item.setToolTipText(toolTip);
		item.setAccelerator(KeyStroke.getKeyStroke(accelerator,
				InputEvent.ALT_MASK));		
		return item;
	}

	private JMenuItem createMenuItem(String iconName, String label,
			String actionCommand, char accelerator) {
		JMenuItem item = createMenuItem(iconName, label, actionCommand);
		item.setAccelerator(KeyStroke.getKeyStroke(accelerator,
				InputEvent.ALT_MASK));
		return item;
	}

	private JCheckBoxMenuItem createMenuCheckBox(String label, String actionCommand){
		JCheckBoxMenuItem item = new JCheckBoxMenuItem(label);
		item.addActionListener(this);
		item.setActionCommand(actionCommand);
		return item;
	}

	private void CompileSelected() {
		JTextArea textArea = getTextArea();
		boolean notNull = (textArea != null);
		compiletoJava.setEnabled(false);
		if (notNull) {
			try {
				LibraryElementView launchView = (LibraryElementView)desktop.getSelectedComponent();
				if (launchView.getDocument().getDocumentElement().getNodeName().equals("FBType")) {
					compiletoJava.setEnabled(true);
				}
			} catch (Exception e) {}
		}
	}

	private void runMenuSelected() {
		JTextArea textArea = getTextArea();
		boolean notNull = (textArea != null);
		runLaunchItem.setEnabled(notNull);		
	}

//	private void DebugSelected() {
//	JTextArea textArea = getTextArea();
//	boolean notNull = (textArea != null);
//	debugItem.setEnabled(notNull);		
//	debugPBItem.setEnabled(notNull);
//	}

	private void editMenuSelected() {
		JTextArea textArea = getTextArea();
		boolean notNull = (textArea != null);
		boolean textSelected = notNull && (textArea.getSelectionEnd() > textArea.getSelectionStart());
		editCutItem.setEnabled(textSelected);
		editCopyItem.setEnabled(textSelected);
		editPasteItem.setEnabled(notNull);
		editDeleteItem.setEnabled(textSelected);
		editFindReplaceItem.setEnabled(notNull);
	}

	/**
	 * Returns the name of a suitable icon for use in menu items as determined
	 * by the suffix of the given file name. If no suitable icon is found,
	 * returns the empty String.
	 */

	public static String getIconForFile(String flnm) {
		int n = flnm.lastIndexOf('.');
		if ((n <= 0) || (n >= flnm.length()))
			return "";
		String suffix = flnm.substring(n);
		String[] sfxs = getFileSuffixes();
		String[] icons = { "fb16", "fb16", "res16", "dev16", "system",
				"file_obj", "xmlObject", "file_obj" };
		for (int i = 0; i < Math.min(sfxs.length, icons.length); i++) {
			if (suffix.equals(sfxs[i]))
				return icons[i];
		}
		return "";
	}

	/** Returns a list of suitable suffixes for IEC 61499 LibraryElement files. */
	public static String[] getFileSuffixes() {
		String[] ans = { ".fbt", ".adp", ".res", ".dev", ".sys", ".dtp",
				".xml", ".src",".db",".trace" };
		return ans;
	}

	/**
	 * Opens the document with the given filename in a new EditorFrame unless
	 * already open; in any case transfers focus to the EditorFrame of the given
	 * file. If none can be found or created, exits with all frames iconified.
	 */
	public void open(String filename) {
		if (show(filename))
			return;
		Document doc = Library.getDocument(filename);
		if (doc == null)
			return;
		LibraryElementView newView = new LibraryElementView(doc);
		newView.getGraphView().setTransferHandler(gvhandler);
		desktop.add(newView);
		desktop.setSelectedComponent(newView);
		CheckToolBar();
	}

	//public void FileOpen(String txt){
	public void FileOpen(Vector v){
		JFileChooser select_item = new JFileChooser("src");
		select_item.setDialogTitle("Open A File (IEC 61499 Compatible)");
		select_item.setFileFilter(new FBLibrary());
		select_item.setFileView(new FBIconLibrary());
		int choice = select_item.showOpenDialog(this);
		if(choice != 0)
			return;
		File f = select_item.getSelectedFile();
		if(f == null)
			return;
		open(f);
	}

	//TODO Name of the file = Name in the first element of the document
	//TODO Location of the file = specified in CompilerInfo - Somehow related with Project?
//	public void fileSaveAsXML(String s) {
	public void fileSaveAsXML(Vector v) {
		LibraryElementView launchView = (LibraryElementView)desktop.getSelectedComponent();

		//((LibraryElementView)desktop.getSelectedComponent()).refresh();
		if(launchView.getDocument() == null){
			JOptionPane.showMessageDialog(this, "The XML document contains errors.",
					"Save failed.", JOptionPane.WARNING_MESSAGE);
			return;
		}

		Document document = launchView.getDocument();

		String extname = getSourceType(launchView);
		String filename = document.getDocumentElement().getAttribute("Name");
		String path = document.getElementsByTagName("CompilerInfo").item(0).getAttributes().getNamedItem("header").getNodeValue();

		Element documentElement = launchView.getDocument().getDocumentElement();
		Element rootElement = documentElement;
		String root = rootElement.getNodeName();
		String contents = new String();
		contents = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		contents = contents + "<!DOCTYPE " + root + " SYSTEM \"http://www.holobloc.com/xml/";
		contents = contents + (root.equals("DataType") ? root : "LibraryElement");
		contents = contents + ".dtd\" >\n";
		contents = contents + XMLCodeGenerator.getXML(documentElement);

		System.out.println(contents);

		new DOMTextModel(launchView.getXmlArea(), document);

		path = removePath(path);
		File f = new File("src\\"+path,filename+extname);

		File directory = new File("src\\"+path);
		if(!directory.exists())
			directory.mkdir();
		String tmpName = "";
		File[] files = directory.listFiles();

		try {
			int bak_ctr = -1;
			int this_bak = 0;
			for(File tmpFile : files){
				tmpName = tmpFile.getName().toUpperCase();
				if(tmpName.contains(filename.toUpperCase()+".BAK")){
					this_bak = Integer.parseInt(tmpName.substring(tmpName.length() - 2));
					if(this_bak > bak_ctr){
						bak_ctr =  this_bak; 
					}
				}
			}
			if (f.exists()) {
				String bak_number = ""+(bak_ctr+1);
				if(bak_number.length() == 1) bak_number = "0"+bak_number; 
				f.renameTo(new File("src\\"+path, filename+".bak"+bak_number));
				f.createNewFile();
			}
			FileWriter fw = new FileWriter(f);
			fw.write(contents);
			fw.close();
		} catch (IOException e) {
			System.err.println(e);
		}

	}


	public String getSourceType(LibraryElementView launchView) {
		String clist = launchView.getDocument().getFirstChild().getNodeName();
		String type = "";
		if (clist.equals("FBType")) {
			type = ".fbt";
		} else if (clist.equals("Device")) {
			type = ".dev";
		} else if (clist.equals("System")) {
			type = ".sys";
		} else if (clist.equals("DataType")) {
			type = ".dtp";
		} else if (clist.equals("AdapterType")) {
			type = ".adp";
		} else if (clist.equals("ResourceType")) {
			type = ".res";
		}
		return type;
	}

//	public void fileSaveAsJava(String s) {
	public void fileSaveAsJava(Vector v) {
		LibraryElementView launchView = (LibraryElementView)desktop.getSelectedComponent();
		int selectedRow = launchView.getSelectedTreeRow(); // Save the selected element so we can put it back
		launchView.setSelectedTreeRow(0); // Select the first treenode so the full source is compiled.
		String filename = launchView.getDocument().getDocumentElement().getAttribute("Name")+".java";
		String path = launchView.getDocument().getElementsByTagName("CompilerInfo").item(0).getAttributes().getNamedItem("header").getNodeValue();
		String contents = launchView.getJavaArea().getText();
		contents = removeHTML(contents);
		path = removePath(path);
		File path_check=new File("rt\\fb\\rt\\"+path);
		if(!path_check.exists()) 
			path_check.mkdir(); 
		File f = new File("rt\\fb\\rt\\"+path,filename);
		try {
			if (f.exists()) { 
				f.delete();
				f.createNewFile();
			}
			FileWriter fw = new FileWriter(f);
			fw.write(contents);
			fw.close();
			Compiler cdlg = new Compiler(this);
			cdlg.showCentered();
			String cmd = adjustCommand(preferences.getProperty("javac") + " -classpath ./rt;./fbrt.jar ");
			cdlg.compile(f, cmd);
		} catch (IOException e) {}
		launchView.setSelectedTreeRow(selectedRow);
	}

	private String adjustCommand(String cmd)
	{
		String ans = cmd.replace(';', File.pathSeparatorChar);
		ans = ans.replace(':', File.pathSeparatorChar);
		ans = ans.replace('\\', File.separatorChar);
		ans = ans.replace('/', File.separatorChar);
		ans = ans.replaceFirst("\\;\\\\", "\\:\\\\");
		return ans;
	}

	public String removePath(String s) {
		int start = s.indexOf("fb.rt.") + 6;
		int end = s.indexOf(";");
		return new String(s.substring(start, end));		
	}

	public String removeHTML(String s) {
		s = s.replace("<br>", "\n");
		s = s.replace("&quot;", "\"");
		s = s.replace("&gt;",">");
		s = s.replace("&lt;","<");
		s = s.replace("&amp;","&");
		int start = s.indexOf("<body>") + 6;
		int end = s.indexOf("</body>");
		return new String(s.substring(start, end));
	}	

	/*	public void debugPB(String s) {
//		if (debugreplayer == null)
//		debugreplayer = new DebugReplayer(s);
//		else if (debugreplayer.getStatus() == 0)
//		debugreplayer = new DebugReplayer(s);
		Component currentComponent = desktop.getSelectedComponent();
		String filename = ((LibraryElementView)currentComponent).getDocument().getDocumentElement().getAttribute("Name");
		File f = new File("rt\\db",filename+".db");
		NodeList clist = ((LibraryElementView)currentComponent).getDocument().getDocumentElement().getChildNodes();
		int type = 0;
		for (int i=0;i<clist.getLength();i++) {
			if (clist.item(i).getNodeName().equals("BasicFB")) {
				type = 1;
				break;
			} else if (clist.item(i).getNodeName().equals("FBNetwork")) {
				type = 2;
				break;
			} else if (clist.item(i).getNodeName().equals("Device")) {
				type = 3;
				break;				
			}
		}
		if (type == 1 | type == 2) {
			if (!f.exists()) {
				try {
					File oldfile = new File("src");
					searchFile newsearch = new searchFile(oldfile.getCanonicalPath());
					Vector list = newsearch.getVector();
					for(int i=0;i<list.size();i++) {
						if (list.get(i).toString().contains(filename+".fbt")) {
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".db");
							break;
						}
					}
				}
				catch (Exception e) {}
			}
//			DebugReplayView docdb = new DebugReplayView(f,type);
//			debugreplayer.createDebugWindow(docdb);
		} else if (type == 3) {
			if (!f.exists()) {
				try {
					File oldfile = new File("src");
					searchFile newsearch = new searchFile(oldfile.getCanonicalPath());
					Vector list = newsearch.getVector();
					for(int i=0;i<list.size();i++) {
						if (list.get(i).toString().contains(filename+".sys")) {
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".db");
							break;
						}
					}
				}
				catch (Exception e) {}
			}
//			DebugReplaySystem docdb = new DebugReplaySystem(f);
//			debugreplayer.createDebugWindow(docdb);			
		}		
	}

	public void debug(String s) {
		if (debugFrame == null)
			debugFrame = new DebugManager();
		else if (debugFrame.getStatus() == 0)
			debugFrame = new DebugManager();
		Component currentComponent = desktop.getSelectedComponent();
		String filename = ((LibraryElementView)currentComponent).getDocument().getDocumentElement().getAttribute("Name");
		File f = new File("rt\\db",filename+".trace");
		NodeList clist = ((LibraryElementView)currentComponent).getDocument().getDocumentElement().getChildNodes();
		int type = 0;
		for (int i=0;i<clist.getLength();i++) {
			if (clist.item(i).getNodeName().equals("BasicFB")) {
				type = 1;
				break;
			} else if (clist.item(i).getNodeName().equals("FBNetwork")) {
				type = 2;
				break;
			} else if (clist.item(i).getNodeName().equals("Device")) {
				type = 3;
				break;				
			}
		}
		if (type == 1 | type == 2) {
			if (!f.exists()) {
				try {
					File oldfile = new File("src");
					searchFile newsearch = new searchFile(oldfile.getCanonicalPath());
					Vector list = newsearch.getVector();
					for(int i=0;i<list.size();i++) {
						if (list.get(i).toString().contains(filename+".fbt")) {
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".db");
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".trace");
							break;
						}
					}
				}
				catch (Exception e) {}
			}
			DebugElementView docdb = new DebugElementView(f,type);
			debugFrame.createDebugWindow(docdb);
		} else if (type == 3) {
			if (!f.exists()) {
				try {
					File oldfile = new File("src");
					searchFile newsearch = new searchFile(oldfile.getCanonicalPath());
					Vector list = newsearch.getVector();
					for(int i=0;i<list.size();i++) {
						if (list.get(i).toString().contains(filename+".sys")) {
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".db");
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".trace");
							break;
						}
					}
				}
				catch (Exception e) {}
			}
			DebugSystemView docdb = new DebugSystemView(f);
			debugFrame.createDebugWindow(docdb);			
		}
	}

	public void debugFBs(String filename) {
		Document tempdoc;
		int type = 0;
		File f1 = new File("rt\\db",filename+".trace");
		try {
			tempdoc = Library.load(new InputSource(new FileInputStream(f1)));
			NodeList cclist = tempdoc.getDocumentElement().getChildNodes();
			for (int i=0;i<cclist.getLength();i++) {
				if (cclist.item(i).getNodeName().equals("BasicFB")) {
					type = 1;
					break;
				} else if (cclist.item(i).getNodeName().equals("FBNetwork")) {
					type = 2;
					break;
				}
			}
		} catch (FileNotFoundException e) {
			System.out.println("ERR: Cannot find file: "+filename+".trace!");
		}

		if (type == 1 | type == 2) {
			if (!f1.exists()) {
				try {
					File oldfile = new File("src");
					searchFile newsearch = new searchFile(oldfile.getCanonicalPath());
					Vector list = newsearch.getVector();
					for(int i=0;i<list.size();i++) {
						if (list.get(i).toString().contains(filename+".fbt")) {
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".db");
							copyFile(list.get(i).toString(),"rt\\db\\"+filename+".trace");
							break;
						}
					}
				}
				catch (Exception e) {}
			}
			DebugElementView docdb = new DebugElementView(f1,type);
			debugFrame.createDebugWindow(docdb);
		}
	}

	public void copyFile(String oldPath,String newPath)  {  
		try {  
			int bytesum = 0;  
			int byteread = 0;  
			File oldfile = new File(oldPath);  
			if (oldfile.exists()) {  
				InputStream inStream = new FileInputStream(oldPath);    
				FileOutputStream fs = new FileOutputStream(newPath);  
				byte[] buffer = new byte[1444];  
				while((byteread=inStream.read(buffer))!=-1){  
					bytesum += byteread;    
					fs.write(buffer,0,byteread);  
				}  
				inStream.close();  
			}  
		}  
		catch  (Exception  e)  {  
			e.printStackTrace();  
		}  
	}
	 */
	public void launch(String cmd) {
		try{
			Process p = Runtime.getRuntime().exec(cmd);
			p.waitFor();
			processes.put(cmd, p);
		}
		catch(IOException iox){
			String msg[] = {"Can't open the Java runtime."};
			JOptionPane.showMessageDialog(this, msg, "Can't open Java runtime", 2);
		}
		catch(InterruptedException e) {}
	}

	public void teminate(Process p){
		p.destroy();
		p = null;
	}

	public void runLaunch(Vector v) {
		LibraryElementView launchView = (LibraryElementView)desktop.getSelectedComponent();
		String keyword = launchView.getDocument().getDocumentElement().getNodeName();
		String file = launchView.getDocument().getDocumentElement().getAttribute("Name");
		if (keyword.equals("FBType"))  
			file = file+".fbt";
		else if (keyword.equals("System"))
			file = file+".sys";
		else {
			System.out.println("UNSUPPORTED TYPE FOR LAUNCH");
		}
		try {
			File f = new File("src");
			SearchFile newsearch = new SearchFile(f.getCanonicalPath());
			Vector list = newsearch.getVector();
			for(int i=0;i<list.size();i++) {
				if (list.get(i).toString().contains(file)) {
					FBJPreferences.loadPrefs("launcher.ini");
					FileLibrary.initStatic();
					FBELibrarySpec.library = new FileLibrary();
					SystemLogger logger = new SystemLogger();
					SystemManager mgr = new SystemManager(logger);
					try{
						FBELibrarySpec spec = FBELibrarySpec.load(new FileReader(list.get(i).toString()), true);
						mgr.launch(spec, FBJPreferences.getBounds());
					}
					catch(Exception e){
						logger.log(e);
						System.exit(-1);
					}
					break;
				}
			}
		}
		catch (Exception e) {}
	}

	public void log(String s){}

	public void log(Exception e){}


	/**
	 * Opens a document from the given file in a new EditorFrame unless already
	 * open; in any case transfers focus to the EditorFrame of the given file.
	 * If none can be found or created, exits with all frames iconified.
	 */
	public void open(File f) {
		if (!f.isFile()) {
			JOptionPane.showMessageDialog(this, f.getAbsolutePath()
					+ " is not a File.", "No Such File",
					JOptionPane.WARNING_MESSAGE);
			return;
		}
		String flnm = f.getName();
		if (show(flnm))
			return;
		try {
			Library.load(new InputSource(new FileInputStream(f)));
			open(flnm);
		} catch (FileNotFoundException e) {}
	}		


	public void createViewFrame() {
		LibraryTreeNode libraryRoot = new LibraryTreeNode(new File("src"), null);
		final JTree libraryTree = new JTree(libraryRoot);
		gvhandler = new GraphViewTransfer(libraryTree);
		libraryTree.setCellRenderer(libraryRoot);
		libraryTree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		libraryTree.setDragEnabled(true);
		libraryTree.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					Object obj = libraryTree.getLeadSelectionPath()
					.getLastPathComponent();
					if (obj instanceof LibraryTreeNode)
						open(((LibraryTreeNode) obj).getFile());
				} else if (e.getButton() == 3) {
					refresh_view.setLocation(e.getX()+viewFrame.getX(), e.getY()+viewFrame.getY());
					refresh_view.setVisible(true);
				} else {
					refresh_view.setVisible(false);					
				}
			}
		});
		JScrollPane libraryView = new JScrollPane(libraryTree);
		viewFrame.setIconImage(IconFactory.getIcon("fbench24.png").getImage());
		viewFrame.setBounds(900, 100, 250, 600);
		viewFrame.getContentPane().add(libraryView);
		viewFrame.setVisible(true);
	}

	public void reloadLibraryTree(Vector v) {
		refresh_view.setVisible(false);
		viewFrame.dispose();
		createViewFrame();
	}

	public void runViewMenu(Vector v) {
		if (viewFrame.getContentPane().getComponentCount() == 0 && treeViewMenu.getState()) {
			createViewFrame();
		} else if (treeViewMenu.getState()) {
			viewFrame.setVisible(true);			
		} else if (!treeViewMenu.getState()) {
			viewFrame.setVisible(false);	
		}
	}

	public void menuDeselected(MenuEvent e) {
	}

	public void menuCanceled(MenuEvent e) {
	}

//	public void editAction(String lbl) {
	public void editAction(Vector v) {
		String lbl = (String) v.get(0);
		JTextArea textarea = getTextArea();
		if (textarea == null)
			return;
		if (lbl.equals("Cut")) {
			textarea.cut();
		} else if (lbl.equals("Copy")) {
			textarea.copy();
		} else if (lbl.equals("Paste")) {
			textarea.paste();
		} else if (lbl.equals("Delete")) {
			textarea.replaceSelection("");
		} else if (lbl.startsWith("Find")) {
			System.out.println("Find/Replace");
		}
	}

	/*	public void EventChanged(DebugEvent event) {
		debugFBs(event.getdebugname());
	}*/

	/**
	 * If an EditorFrame is selected and not iconified, return its text area;
	 * otherwise return <TT>null</TT>.
	 */
	public JTextArea getTextArea() {
		Component fr = desktop.getSelectedComponent();
		return (fr == null) ? null : ((LibraryElementView) fr).getXmlArea();
	}

	public JTabbedPane getDesktop(){
		return desktop;
	}

	public Hashtable<String, Object> getInstances(){
		return instances;
	}

	private void setupPreferences() {
		try {
			preferences.load(new FileInputStream("preferences.ini"));
		} catch (IOException e) {
			System.out.println("Error reading preferences: " + e.getMessage());
		}
		//preferences.list(System.out);
	}

	public void newCFB(Vector v){
		open("composite.fbt");
	}

	public void newBFB(Vector v){
		open("Basic.fbt");
	}

	public void newProxy(Vector v){
		open("Proxy.fbt");
	}

	public void newSI(Vector v){
		open("ServiceInterface.fbt");
	}

	public void newSimple(Vector v){
		open("Simple.fbt");
	}

	public void newAdapt(Vector v){
		open("AdapterType.adp");
	}

	public void newRes(Vector v){
		open("ResourceType.res");
	}

	public void newDev(Vector v){
		open("DeviceType.dev");
	}

	public void newSys(Vector v){
		open("System.sys");
	}

	public void newArray(Vector v){
		open("Array.dtp");
	}

	public void newDirect(Vector v){
		open("Direct.dtp");
	}

	public void newEnu(Vector v){
		open("Enumerated.dtp");
	}

	public void newStruct(Vector v){
		open("Structured.dtp");
	}

	public void newSubrange(Vector v){
		open("Subrange.dtp");
	}

	public int help(Vector v) {
		String topic = (String) v.get(0);
		if (topic.startsWith("About")) {
			JLabel msg = new JLabel(
					"<HTML><CENTER><P>"
					+ FBench.productID
					+ " Version "
					+ FBench.version
					+ "<BR><BR>Open Source Project of OOONEIDA"
					+ "<BR>Developed at The University of Auckland"
					+ "<BR><A HREF=\"http://www.ece.auckland.ac.nz\">Department of Electrical and Computer Engineering</A></P>"
					+ "<BR>Intelligent Mechatronics and Automation Group</A></P>"
					+ "<BR><BR>By clicking &quot;OK&quot; you agree to use this software"
					+ "<BR>under the provisions of the Common Public License"
					+ "<BR><A HREF=\"http://www.opensource.org/cpl1.0.php\">&lt;http://www.opensource.org/cpl1.0.php&gt;</A>."
					+ "</CENTER></HTML>");
			msg.setBackground(Color.lightGray);
			return JOptionPane.showConfirmDialog(this, msg, "About "
					+ FBench.productID, JOptionPane.PLAIN_MESSAGE,
					JOptionPane.OK_OPTION, IconFactory.getIcon("FBench0.jpg"));
		}
		if (topic.equals("FBench Help"))
			topic = "wb/FBench.htm";
		else if (topic.equals("How To..."))
			topic = "wb/howto.htm";
		String[] cmd = new String[2];
		cmd[0] = preferences.getProperty("browser");
		cmd[1] = (System.getProperty("user.dir") + "/doc/" + topic).replace(
				'/', File.separatorChar);
		try {
			Runtime.getRuntime().exec(cmd);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(),
					"Error opening " + topic, JOptionPane.WARNING_MESSAGE);
		}
		return 0;
	}

	/**
	 * If a file with the given name is already open in a tabbed pane, brings
	 * that tab to the top.
	 * 
	 * @param filename, e.g., <TT>X2Y2.fbt</TT> (the suffix is ignored).
	 * @return <TT>true</TT> if an open file was found.
	 */
	protected boolean show(String filename) {
		int n = filename.indexOf('.');
		if (n > 0)
			filename = filename.substring(0, n);
		for (int i = 0; i < desktop.getComponentCount(); i++) {
			Component c = desktop.getComponent(i);
			if (c.getName().equals(filename)) {
				desktop.setSelectedComponent(c);
				return true;
			}
		}
		return false;
	}

	class ActionSet extends AbstractAction{
		public ActionSet(String name,Icon icon){
			super(name,icon);
		}

		public void actionPerformed(ActionEvent e){
			String command = e.getActionCommand();
			/*
			if(command.equals("Parse XML")){
				((LibraryElementView)desktop.getSelectedComponent()).refresh(); 
			}
			 */
			if(command.equals("Close Tab")){
				if (JOptionPane.showConfirmDialog(null, "Save before close?", "Save before close?", JOptionPane.YES_NO_OPTION)==0) {
					Vector args = new Vector();
					args.add(desktop.getSelectedComponent().getName());
					fileSaveAsXML(args);
				}
				desktop.remove(desktop.getSelectedComponent());
				CheckToolBar();
			}
			if(command.equals("Save Now")){
				Vector args = new Vector();
				args.add(desktop.getSelectedComponent().getName());
				fileSaveAsXML(args);
			}
			if(command.equals("Next Tab")){
				if (desktop.getSelectedIndex() < desktop.getComponentCount() - 1)
					desktop.setSelectedIndex(desktop.getSelectedIndex()+1);
			}
			if(command.equals("Previous Tab")){
				if (desktop.getSelectedIndex() > 0)
					desktop.setSelectedIndex(desktop.getSelectedIndex()-1);
			}
		}	        
	}
}