//Copyright (c)2005 Holobloc Inc. All rights reserved.
//Contributed to the OOONEIDA FBench project under the Common Public License.

//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.text.html.HTMLEditorKit;

import org.oooneida.wb.dom.DOMTextModel;
import org.oooneida.wb.dom.DOMTranslationModel;
import org.oooneida.wb.graph.GraphView;
import org.oooneida.wb.table.ElementTableModel;
import org.oooneida.wb.translate.STtoJava;
import org.oooneida.wb.translate.TranslatorSource;
import org.oooneida.wb.tree.DOMTree;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A panel for navigating graphical and textual views of an IEC 61499
 * LibraryElement.
 * 
 * @author JHC, DH, JP
 * 
 * @version 20070301/JP - Removed unused code and moved some segment of codes to other classes
 * @version 20070131/DH - Found bugs from  WD, AS previous works.  Fixed the bugs and 
 *                        created extra dialogs for Adding, Modifying and Removing various graphical 
 * 							components
 * @version 20061131/WD,AS - Project Manager, Dialog for adding,modifying and removing interface elements 
 * @version 20051109/JHC - All notifications now via DOM Event mechanism.
 * @version 20050418/JHC
 */

public class LibraryElementView extends JPanel{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 3395261111185224686L;

	private Document document;

	int mouseX, mouseY;

	private GraphView graphView;

	private DOMTree tree;

	private JTextArea xmlArea = new JTextArea();

	private JSplitPane mainPanel;

	private JEditorPane srcArea = new JEditorPane();

	private JEditorPane javaArea = new JEditorPane();

	private JTabbedPane tabbedPane;

	private Color selectedColor = new Color(119, 190, 255);
	
	private TranslatorSource translatorSource;

	public LibraryElementView(Document doc) {
		super();
		Element docel = doc.getDocumentElement();
		setName(docel.getAttribute("Name"));
		document = doc;
		this.translatorSource = new TranslatorSource();
		translatorSource.addTranslator(STtoJava.class, "ST", "Java");
		initComponents();
	}

	private void initComponents() {
		tree = new DOMTree(document);
		
		ElementTableModel tblmdl = new ElementTableModel(document);
		JTable props = new JTable(tblmdl);
		props.setSelectionBackground(selectedColor);
		props.setFont(new Font("SansSerif", Font.PLAIN, 11));
		JScrollPane propsView = new JScrollPane(props);
		propsView.setPreferredSize(new Dimension(400, 150));

		JScrollPane treeView = new JScrollPane(tree);
		treeView.setPreferredSize(new Dimension(150, 200));
//		JPanel rightPanel = new JPanel(new BorderLayout());
//		rightPanel.add(treeView, BorderLayout.CENTER);
		JSplitPane rightPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
				treeView, propsView);

		graphView = new GraphView(document);

		JScrollPane graphicView = new JScrollPane(graphView);
		graphicView.setPreferredSize(new Dimension(450, 200));

		JSplitPane topPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
				graphicView, rightPane);
		topPane.setResizeWeight(1.0);
		topPane.setDividerLocation(450);
		topPane.setOneTouchExpandable(true);
		
		tabbedPane = createTabs();

		mainPanel = update(topPane, tabbedPane);
		mainPanel.setResizeWeight(1.0);
		setLayout(new BorderLayout());
		add("Center", mainPanel);
		tree.setSelectionRow(0);

	}

	public JTabbedPane createTabs(){
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.setFont(new Font("SansSerif", Font.PLAIN, 10));
		xmlArea.setSelectionColor(selectedColor);
		xmlArea.setEditable(true);
		xmlArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		new DOMTextModel(xmlArea, document);
		xmlArea.setCaretPosition(0);
		JScrollPane xmlView = new JScrollPane(xmlArea);

		xmlView.setPreferredSize(new Dimension(200, 150));
		tabbedPane.add("XML", xmlView);        

		srcArea.setSelectionColor(selectedColor);
		srcArea.setEditable(false);
		srcArea.setEditorKit(new HTMLEditorKit());
		new DOMTranslationModel(document,srcArea,"DOMtoSRC.xsl", "SRC", translatorSource);
		//TODO Complete DOMtoSRC.xls
		JScrollPane srcView = new JScrollPane(srcArea);
		srcView.setPreferredSize(new Dimension(400, 150));
		tabbedPane.add("Src", srcView);

		javaArea.setSelectionColor(selectedColor);
		javaArea.setEditable(false);
		javaArea.setEditorKit(new HTMLEditorKit());
		new DOMTranslationModel(document,javaArea,"DOMtoJava.xsl", "Java", translatorSource);

		//TODO Complete DOMtoJava.xls
		JScrollPane jView = new JScrollPane(javaArea);
		jView.setPreferredSize(new Dimension(400, 150));
		tabbedPane.add("Java", jView);

		return tabbedPane;
	}


	public JSplitPane updateTop(JScrollPane a, JPanel b){
		return new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, a, b);
	}
	
	public JSplitPane update(JSplitPane a, JTabbedPane b){
		return new JSplitPane(JSplitPane.VERTICAL_SPLIT, a, b);
	}

	public JTextArea getXmlArea() {
		return xmlArea;
	}

	public GraphView getGraphView() {
		return graphView;
	}

	public GraphView getGraphView(Document doc) {
		return graphView = new GraphView(doc);
	}
	
	public void setSelectedTreeRow(int row){
		tree.setSelectionRow(row);
	}
	
	public int getSelectedTreeRow(){
		return tree.getSelectionRows()[0];
	}
/*
	public void refresh() {
		try{
			// get edited data from xmlArea into a temporary file called temp.fbt
			FileOutputStream fo1 = new FileOutputStream ("temp.fbt");
			PrintStream ps1 = new PrintStream(fo1);
			ps1.print(xmlArea.getText()); 
			//Use the factory to create a DOM parser (a.k.a. a DocumentBuilder) 
			try {
				document = Library.load(new InputSource(new FileInputStream("temp.fbt")));
			} catch(FileNotFoundException ex) {}        
			NodeList e1 = document.getElementsByTagName("FBType");
			graphView.setElement((Element)(e1.item(0)));

		}
		catch(Exception ex){
			System.out.println(ex.toString());
		};	
	}{}
*/
	public JEditorPane getJavaArea() {
		return javaArea;
	}
	public Document getDocument() {
		return document;
	}
	public DOMTree getTree(){
		return tree;
	}
}
