//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.dom;

import java.awt.Point;
import java.util.Hashtable;

import javax.swing.JTextArea;

import org.oooneida.wb.XMLCodeGenerator;
import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * A model for displaying and navigating in a JTextArea the XML source code of
 * an IEC 61499 LibraryElement whose underlying model is a DOM Document.
 * 
 * @author JHC
 * @version 20051115/JHC - Made a DOM EventListener displaying only selected
 *          Element.
 * @version 20050420/JHC
 */
public class DOMTextModel extends ElementModel implements EventListener {
	protected JTextArea textarea;
	protected Hashtable<Element, Point> selectionRanges;
	protected Document document;

	public DOMTextModel(JTextArea textArea, Document document) {
		super();
		this.textarea = textArea;
		this.selectionRanges = new Hashtable<Element, Point>();
		setDocument(document);
		
	}

	/** Set the document and refresh the display. */
	public void setDocument(Document doc) {
		this.document = doc;
		this.selectionRanges.clear();
		((org.w3c.dom.events.EventTarget) doc).addEventListener("ElementSelectionEvent", this,
				false);
		textarea.setText("");
		textarea.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		Element rootElement = document.getDocumentElement();
		String root = rootElement.getNodeName();
		textarea.append("<!DOCTYPE " + root
//				TODO: change to auck uni website?
				+ " SYSTEM \"http://www.holobloc.com/xml/");
		textarea.append(root.equals("DataType") ? root : "LibraryElement");
		textarea.append(".dtd\" >\n");
		write(rootElement, "");
	}

	/**
	 * Write out the given DOM Element and its children to the text area with
	 * the given indentation.
	 */
	private void write(Element el, String indent) {
		Point p = new Point();
		textarea.append(indent);
		p.x = textarea.getText().length();
		textarea.append("<");
		String nname = el.getNodeName();
		textarea.append(nname);
		NamedNodeMap atts = el.getAttributes();
		int n = atts.getLength();
		if (n > 0) {
			for (int i = 0; i < n; i++) {
				Attr att = (Attr) atts.item(i);
				textarea.append(" ");
				textarea.append(att.getName());
				textarea.append("=\"");
				String val = att.getValue();
				val = XMLCodeGenerator.replaceReservedCharacters(val);
				textarea.append(val);
				textarea.append("\"");
			}
		}
		NodeList nodes = el.getChildNodes();
		n = nodes.getLength();
		if (n == 0) {
			textarea.append("/>\n");
			p.y = textarea.getText().length() - 1;
		} else {
			textarea.append(">\n");
			p.y = textarea.getText().length() - 1;
			String newindent = indent + "   ";
			for (int i = 0; i < n; i++)
				write((Element) nodes.item(i), newindent);
			textarea.append(indent);
			textarea.append("</");
			textarea.append(nname);
			textarea.append(">\n");
		}
		selectionRanges.put(el, p);
	}

	public void handleEvent(Event evt) {
		if (evt instanceof MutationEvent)
			setElement(getElement());
		else if(evt instanceof ElementSelectionEvent){
			Element e = ((ElementSelectionEvent)evt).getSelectedElement();
			Point p = selectionRanges.get(e);
			textarea.requestFocus();
			textarea.setSelectionStart(p.y);
			textarea.setSelectionEnd(p.x);
		}
	}

	public void setElement(Element el) {
		super.setElement(el);
		((org.w3c.dom.events.EventTarget) el).addEventListener("MutationEvent", this, false);
		if (el == document.getDocumentElement())
			setDocument(document);
		else {
			textarea.setText("");
			write(el, "");
		}
	}
}
