<?xml version="1.0" encoding="ISO-8859-1"?>
<!-- Copyright (c)2005 Holobloc Inc. -->
<!-- Contributed to the OOONEIDA Workbench project under the Common Public License. -->

<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  version="1.0"
  >
<xsl:output method="html"/>
<!-- Document Template -->
<xsl:template match="/">
    <html><body><CODE>
    <xsl:apply-templates/>
    </CODE></body></html>
</xsl:template>
  
<!-- Utility templates -->
<xsl:template name="indent4">
   <xsl:call-template name="indent2"/>
   <xsl:call-template name="indent2"/>
</xsl:template>

<xsl:template name="indent2">
   <xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;</xsl:text>
</xsl:template>

<xsl:template name="break">
   <xsl:text disable-output-escaping="yes">&lt;BR&gt;</xsl:text>
</xsl:template>

<xsl:template name="name-and-comment">
   <xsl:value-of select="@Name"/><xsl:call-template name="comment"/>
</xsl:template>

<xsl:template name="comment">
   <xsl:if test="@Comment"><FONT COLOR="green"> (* <xsl:value-of select="@Comment"/> *)</FONT></xsl:if>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template name="type">
   <xsl:if test="@Type"> : <xsl:value-of select="@Type"/></xsl:if>
</xsl:template>

<xsl:template name="comment-params">
   <xsl:param name="terminator"/>
   <xsl:choose>
     <xsl:when test="Parameter">(
       <xsl:call-template name="comment"/>
       <xsl:apply-templates select="Parameter">
         <xsl:with-param name="separator">,</xsl:with-param>
         <xsl:with-param name="terminator">)<xsl:value-of select="$terminator"/></xsl:with-param>
       </xsl:apply-templates>
     </xsl:when>
     <xsl:otherwise>
       <xsl:value-of select="$terminator"/><xsl:call-template name="comment"/>
     </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<!-- Top Level Templates -->
<xsl:template match="/FBType">
   <FONT color="blue">FUNCTION_BLOCK </FONT><xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_FUNCTION_BLOCK</FONT>
</xsl:template>

<xsl:template match="/System">
   <FONT color="blue">SYSTEM </FONT><xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates select="Device"/>
   <xsl:if test="Segment">
     <FONT color="blue">SEGMENTS</FONT>
     <xsl:call-template name="break"/>
     <xsl:apply-templates select="Segment"/>
     <FONT color="blue">END_SEGMENTS</FONT>
     <xsl:call-template name="break"/>
   </xsl:if>
   <xsl:if test="Link">
     <FONT color="blue">LINKS</FONT>
     <xsl:call-template name="break"/>
     <xsl:apply-templates select="Link"/>
     <FONT color="blue">END_LINKS</FONT>
     <xsl:call-template name="break"/>
   </xsl:if>
   <FONT color="blue">END_SYSTEM</FONT>
</xsl:template>

<xsl:template match="/DeviceType">
   <FONT color="blue">DEVICE_TYPE </FONT><xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_DEVICE_TYPE</FONT>
</xsl:template>

<xsl:template match="/ResourceType">
   <FONT color="blue">RESOURCE_TYPE </FONT><xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_RESOURCE_TYPE</FONT>
</xsl:template>

<xsl:template match="/AdapterType">
   <FONT color="blue">ADAPTER </FONT><xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_ADAPTER</FONT>
</xsl:template>

<!-- Event Interface -->
<xsl:template match="EventInputs">
   <FONT color="blue">EVENT_INPUT</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_EVENT</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="EventOutputs">
   <FONT color="blue">EVENT_OUTPUT</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_EVENT</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="Event">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:if test="With">
      <FONT color="blue"> WITH </FONT> 
      <xsl:for-each select="With">
         <xsl:value-of select="@Var"/>
         <xsl:if test="not(position()=last())">, </xsl:if>
      </xsl:for-each>
   </xsl:if>;
   <xsl:call-template name="comment"/>
</xsl:template>

<!-- Variable Interface -->
<xsl:template match="InputVars">
   <FONT color="blue">VAR_INPUT</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_VAR</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="OutputVars">
   <FONT color="blue">VAR_OUTPUT</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_VAR</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="VarDeclaration">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:if test="@ArraySize">[<xsl:value-of select="@ArraySize"/>]</xsl:if>
   <xsl:if test="@InitialValue"> := <xsl:value-of select="@InitialValue"/></xsl:if>
   ;<xsl:call-template name="comment"/>
</xsl:template>

<!-- FBNetworks -->
<xsl:template match="FBNetwork">
   <FONT color="blue">FBS</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates select="FB"/>
   <FONT color="blue">END_FBS</FONT>
   <xsl:call-template name="break"/>
   <xsl:apply-templates select="EventConnections"/>
   <xsl:apply-templates select="DataConnections"/>
   <xsl:apply-templates select="AdapterConnections"/>
</xsl:template>

<xsl:template match="FB">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:call-template name="comment-params">
     <xsl:with-param name="terminator">;</xsl:with-param>
   </xsl:call-template>
</xsl:template>

<xsl:template match="Parameter">
  <xsl:param name="separator"/>
  <xsl:param name="terminator"/>
  <xsl:call-template name="indent4"/>
  <xsl:value-of select="@Name"/> := <xsl:value-of select="@Value"/>
  <xsl:if test="not(position()=last())"><xsl:value-of select="$separator"/></xsl:if>
  <xsl:if test="position()=last()"><xsl:value-of select="$terminator"/></xsl:if>
  <xsl:call-template name="comment"/>
</xsl:template>

<xsl:template match="EventConnections">
   <FONT color="blue">EVENT_CONNECTIONS</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_CONNECTIONS</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="DataConnections">
   <FONT color="blue">DATA_CONNECTIONS</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_CONNECTIONS</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="AdapterConnections">
   <FONT color="blue">ADAPTER_CONNECTIONS</FONT><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_CONNECTIONS</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="Connection">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Source"/>
   <xsl:text > TO </xsl:text>
   <xsl:value-of select="@Destination"/> ;
   <xsl:call-template name="comment"/>
</xsl:template>

<!-- ECC elements -->
<xsl:template match="ECC">
   <xsl:text>EC_STATES</xsl:text><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <xsl:text>END_STATES</xsl:text><xsl:call-template name="break"/>
   <xsl:text>EC_TRANSITIONS</xsl:text><xsl:call-template name="break"/>
   <xsl:apply-templates/>
   <xsl:text>END_TRANSITIONS</xsl:text><xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="ECState">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Name"/>
   <xsl:choose>
     <xsl:when test="ECAction">:
       <xsl:call-template name="comment"/>
       <xsl:apply-templates />
     </xsl:when>
     <xsl:otherwise>;
       <xsl:call-template name="comment"/>
     </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="ECAction">
  <xsl:call-template name="indent4"/>
  <xsl:value-of select="@Algorithm"/>
  <xsl:if test="@Output"> -&gt; </xsl:if>
  <xsl:value-of select="@Output"/>
  <xsl:if test="not(position()=last())">,</xsl:if>
  <xsl:if test="position()=last()">;</xsl:if>
  <xsl:call-template name="comment"/>
</xsl:template>

<!-- Service elements -->
<xsl:template match="Service">
   <FONT color="blue">SERVICE </FONT>
   <xsl:value-of select="@LeftInterface"/>/<xsl:value-of select="@RightInterface"/>
   <xsl:call-template name="comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_SERVICE</FONT>
   <xsl:call-template name="break"/>
</xsl:template>
<xsl:template match="ServiceSequence">
   <FONT color="blue">SEQUENCE </FONT>
   <xsl:call-template name="name-and-comment"/>
   <xsl:apply-templates/>
   <FONT color="blue">END_SEQUENCE</FONT>
   <xsl:call-template name="break"/>
</xsl:template>
<xsl:template match="ServiceTransaction">
   <xsl:apply-templates/>;
   <xsl:call-template name="break"/>
</xsl:template>
<xsl:template match="InputPrimitive">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Interface"/>.<xsl:value-of select="@Event"/>(<xsl:value-of select="@Parameters"/>)
</xsl:template>
<xsl:template match="OutputPrimitive">
   <FONT color="blue"><xsl:text> -&gt; </xsl:text></FONT>
   <xsl:value-of select="@Interface"/>.<xsl:value-of select="@Event"/>(<xsl:value-of select="@Parameters"/>)
</xsl:template>

<!-- System configuration elements -->
<xsl:template match="Device">
   <FONT color="blue">DEVICE </FONT>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:call-template name="comment-params"/>
   <xsl:apply-templates select="Resource"/>
   <xsl:apply-templates select="FBNetwork"/>
   <FONT color="blue">END_DEVICE</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<xsl:template match="Resource">
   <FONT color="blue">RESOURCE </FONT>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:call-template name="comment-params"/>
   <xsl:apply-templates select="FBNetwork"/>
   <FONT color="blue">END_RESOURCE</FONT>
   <xsl:call-template name="break"/>
</xsl:template>

<!-- Network Elements -->
<xsl:template match="Segment">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@Name"/>
   <xsl:call-template name="type"/>
   <xsl:call-template name="comment-params">
     <xsl:with-param name="terminator">;</xsl:with-param>
   </xsl:call-template>
</xsl:template>

<xsl:template match="Link">
   <xsl:call-template name="indent2"/>
   <xsl:value-of select="@SegmentName"/>
   <xsl:text disable-output-escaping="yes">=&amp;gt;</xsl:text>
   <xsl:value-of select="@CommResource"/>
   <xsl:call-template name="comment-params">
     <xsl:with-param name="terminator">;</xsl:with-param>
   </xsl:call-template>
</xsl:template>

</xsl:stylesheet>
