//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.dom.events;

import org.oooneida.wb.dom.ElementModel;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * An ElementModel that listens for MutationEvents from its Element (which may
 * include events that have bubbled up from the Element's children).
 * 
 * @author JHC
 * @version 20051111/JHC
 */
public abstract class MutationListener extends ElementModel implements
        EventListener {

    public MutationListener() {
        super();
    }

    public MutationListener(Element el) {
        super(el);
    }

    public void setElement(Element el) {
        super.setElement(el);
        if (element != null){
            ((org.w3c.dom.events.EventTarget) element).addEventListener("MutationEvent", this, false);
        }
    }

    public void handleEvent(Event evt) {
        if (!(evt instanceof MutationEvent))
            return;
        MutationEvent mutevt = (MutationEvent) evt;
        Node node = mutevt.getRelatedNode();
        if (!(node instanceof Element))
            return;
        Element el = (Element) node;
        switch (mutevt.getAttrChange()) {
            case MutationEvent.MODIFICATION: {
                handleModification(mutevt, el);
                return;
            }
            case MutationEvent.ADDITION: {
                handleAddition(mutevt, el);
                return;
            }
            case MutationEvent.REMOVAL: {
                handleRemoval(mutevt, el);
            }
        }
    }

    /**
     * Handle a MutationEvent of MODIFICATION type on the given Element. Default
     * is to process a change (if any) in the <TT>Name</TT> of the referenced
     * element.
     */
    protected void handleModification(MutationEvent mutev, Element el) {
        if (mutev.getAttrName().equals("Name"))
            processNameChange(mutev, el);
    }

    /**
     * Handle a MutationEvent of ADDITION type on the given Element. Default is
     * to do nothing.
     */
    protected void handleAddition(MutationEvent mutev, Element el) {
    }

    /**
     * Handle a MutationEvent of REMOVAL type on the given Element. Default is
     * to do nothing.
     */
    protected void handleRemoval(MutationEvent mutev, Element el) {
    }

    /**
     * Returns an <TT>ElementModel</TT> in the <TT>"dom."</TT> package
     * encapsulating the given DOM Element, or <TT>null</TT> if no such
     * ElementModel exists.
     * 
     * @param el The DOM Element for which a model is requested.
     */
    public static MutationListener forElement(Element el) {
        ElementModel ans = ElementModel.forElement(el,
                "org.oooneida.wb.dom.events.");
        return (ans instanceof MutationListener) ? (MutationListener) ans
                : null;
    }

    /**
     * Process a change in the <TT>Name</TT> attribute of the referenced
     * element.
     */
    protected abstract void processNameChange(MutationEvent evt, Element el);
}
