//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.

//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Area;
import java.awt.geom.RoundRectangle2D;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

import org.oooneida.wb.IconFactory;
import org.oooneida.wb.LibraryElementView;
import org.oooneida.wb.FBench;
import org.oooneida.wb.graph.model.GraphModel;
import org.oooneida.wb.graph.popup.ParameterDialog;
import org.oooneida.wb.tree.DOMTree;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A graphic element that encapsulates an ElementModel for an IEC 61499 <TT>
 * FBType</TT> library Element.
 * 
 * @author JHC
 * @version 20051109/JHC - Removed ItemListener implementation - all selection
 *          now done via ElementSelectionEvent.
 * @version 20051027/JHC - No topRect or midRect in updateShape() if no event
 *          I/O.
 * @version 20051025/JHC - Set up Plugs, Sockets.
 * @version 20050817/JHC
 */
public class FBType extends GraphNode {
    protected static final Insets leftInsets = new Insets(1, 0, 1, 3);

    protected static final Insets rightInsets = new Insets(1, 3, 1, 0);

    protected RoundRectangle2D topRect = new RoundRectangle2D.Float();

    protected Rectangle midRect = new Rectangle();

    protected RoundRectangle2D bottomRect = new RoundRectangle2D.Float();

    protected int prefx = 0;

    protected boolean reversed = false;

    public FBType() {
        super();
        setLayout(new GridBagLayout());
        mainLabel.setOpaque(false);
    }

    protected void initComponents() {
        super.initComponents();
        prefx = 0;
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridwidth = 1;
        gbc.weightx = 0.5;
        gbc.fill = GridBagConstraints.NONE;
        NodeList intfclst = getElement().getElementsByTagName("InterfaceList");
        if (intfclst.getLength() < 1)
            return;
        Element intfcs = (Element) intfclst.item(0);
        int eict = addInterfaceElements(intfcs, reversed ? "EventOutputs"
                : "EventInputs", gbc);
        gbc.gridx = 1;
        gbc.gridy = 0;
        int eoct = addInterfaceElements(intfcs, reversed ? "EventInputs"
                : "EventOutputs", gbc);
        gbc.gridx = 0;
        int lblrow = Math.max(eict, eoct);
        gbc.gridy = lblrow;
        gbc.gridwidth = 2;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = new Insets(getFontMetrics(getFont()).getHeight(), 2, 2, 2);
        mainLabel.addMouseListener(new MouseAdapter(){            
        	public void mouseClicked(MouseEvent evt) {
        		me.processMousePress();
        		if(evt.getButton() == MouseEvent.BUTTON1 && evt.getClickCount() == 2){
        			try{
        				Container parent = getParent();
        				while(!parent.getClass().toString().endsWith("org.oooneida.wb.FBench")){
        					parent = parent.getParent();
        				}
        				FBench FBench = (FBench)parent;
        				FBench.open(mainLabel.getText());
        			}
        			catch (NullPointerException e){	}
        		}
        	}
        });
        add(mainLabel, gbc);
        gbc.gridy = lblrow + 1;
        gbc.gridwidth = 1;
        addInterfaceElements(intfcs, reversed ? "OutputVars" : "InputVars", gbc);
        addInterfaceElements(intfcs, reversed ? "Plugs" : "Sockets", gbc);
        gbc.gridx = 1;
        gbc.gridy = lblrow + 1;
        addInterfaceElements(intfcs, reversed ? "InputVars" : "OutputVars", gbc);
        addInterfaceElements(intfcs, reversed ? "Sockets" : "Plugs", gbc);
    }

    /**
     * Adds a set of interface elements and returns the number so added.
     * 
     * @param intfcs The <TT>InterfaceList</TT> element from which to retrieve
     *            the interface elements.
     * @param tag The name of the subset of interface elements to be added,
     *            e.g., <TT>InputVars</TT>.
     * @param gbc The constraints specifying the beginning position for element
     *            insertion (will be modified during the course of element
     *            insertion).
     * @return The index of the next row to add.
     */
    protected int addInterfaceElements(Element intfcs, String tag,
            GridBagConstraints gbc) {
        NodeList lst = intfcs.getElementsByTagName(tag);
        if (lst.getLength() < 1)
            return 1;
        return addInterfaceElements(lst.item(0).getChildNodes(), gbc);
    }

    /**
     * Adds a set of interface elements and returns the number so added.
     * 
     * @param lst The list of interface elements to be added.
     * @return The number of interface elements added.
     */
    protected int addInterfaceElements(NodeList lst, GridBagConstraints gbc) {
        int n = lst.getLength();
        if (n < 1)
            return 0;
        int ha = SwingConstants.LEFT;
        int hta = SwingConstants.RIGHT;
        if (gbc.gridx == 0) {
            gbc.insets = FBType.leftInsets;
            gbc.anchor = GridBagConstraints.WEST;
        } else {
            gbc.insets = FBType.rightInsets;
            gbc.anchor = GridBagConstraints.EAST;
            ha = SwingConstants.RIGHT;
            hta = SwingConstants.LEFT;
        }
        for (int i = 0; i < n; i++) {
            Element el = (Element) lst.item(i);
            String txt = el.getAttribute("Name");
            JLabel lbl = new JLabel(txt);
            lbl.addMouseListener(new MouseAdapter(){    
            	private LibraryElementView libElemView;
            	private Element fb, parameter;
            	private String variable;
            	
            	public void mouseClicked(MouseEvent evt) {
            		me.processMousePress();
            		
            		boolean isDoubleClicked = evt.getButton() == 1 && evt.getClickCount() == 2;
            		boolean isRightClicked = evt.getButton() == 3;
            		if((isDoubleClicked || isRightClicked) && isInFBench()){
            			if(libElemView == null){
            				Container parent = getParent();
            				while(!parent.getClass().toString().endsWith("org.oooneida.wb.LibraryElementView")){
            					parent = parent.getParent();
            				}
            				libElemView = (LibraryElementView)parent;
            			}
            			GraphView graphView = libElemView.getGraphView();
            			
            			variable = ((JLabel)evt.getSource()).getText();
            			fb = ((Element)graphView.getSelectedGraphElement().getElement()); 
            			if(fb.getNodeName().equals("FB")){
            				boolean isSuitableForHavingParameter = false;
            				if(fb.getParentNode().getParentNode().getNodeName().equals("Application") || 
            						fb.getParentNode().getParentNode().getNodeName().equals("Resource"))
            					isSuitableForHavingParameter = true;
            				
            				if(!isSuitableForHavingParameter) return;
            				
            				boolean canHaveParameter = true;
            				String target = fb.getAttribute("Name")+"."+variable;
            				Element fbNetwork = (Element) fb.getParentNode();
            				NodeList connectionNL = fbNetwork.getElementsByTagName("Connection");
            				
            				for(int i = 0; i < connectionNL.getLength(); i++){
            					String destination = ((Element) connectionNL.item(i)).getAttribute("Destination");
            					String source = ((Element) connectionNL.item(i)).getAttribute("Source");
            					if(destination.equals(target) || source.equals(target)){
            						canHaveParameter = false;
            						break;
            					}
            				}
            				
            				if(!canHaveParameter) return;
            			}
            			
            			String gvModel = graphView.getModel().getElement().getNodeName();
            			
            			NodeList parameterNL = fb.getElementsByTagName("Parameter");
    					boolean doesAlreadyHaveParameter = false;
    					for(int i = 0; i < parameterNL.getLength(); i++){
    						if(((Element)parameterNL.item(i)).getAttribute("Name").equals(variable)){
    							doesAlreadyHaveParameter = true;
    							parameter = (Element) parameterNL.item(i);
    						}
    					}
    					if(isDoubleClicked && doesAlreadyHaveParameter && !gvModel.equals("FBType")){
    						ParameterDialog dialog = new ParameterDialog(parameter, evt);
							dialog.create("Edit Parameter");
    					}
    					else if(isDoubleClicked && !doesAlreadyHaveParameter && !gvModel.equals("FBType")){
    						ParameterDialog dialog = new ParameterDialog(fb, evt);
    						dialog.create("Add Parameter."+variable);
    					}
            		}
            	}
            });
            lbl.setOpaque(false);
            lbl.setName(txt);
            lbl.setFont(getFont());
            lbl.setHorizontalAlignment(ha);
            lbl.setHorizontalTextPosition(hta);
            if (el.getTagName().equals("AdapterDeclaration"))
                lbl.setIcon(IconFactory.getIcon("adapter12"));
            add(lbl, gbc);
            gbc.gridy++;
        }
        return gbc.gridy;
    }

    protected String getTypeSuffix() {
        return ".fbt";
    }

    protected void updateShape() {
        outline.reset();
        int w = getWidth() - 1;
        int y2 = mainLabel.getY();
        if (hasIO("EventInputs") || hasIO("EventOutputs")) {
            int y0 = 0;
            int y1 = y2 - mainLabel.getHeight();
            topRect.setRoundRect(0, y0, w, y1 - y0, 8, 8);
            outline.add(new Area(topRect));
            midRect.setBounds(getWidth() / 4, y1, getWidth() / 2, y2 - y1);
            outline.add(new Area(midRect));
        }
        bottomRect.setRoundRect(0, y2, w, getHeight() - y2 - 1, 8, 8);
        outline.add(new Area(bottomRect));
    }

    public Rectangle getPreferredBounds() {
        int y = getFontMetrics(getFont()).getHeight();
        Dimension sz = getPreferredSize();
        return new Rectangle(prefx, y, sz.width, sz.height);
    }

    /** Sets the x-coordinate for <TT>getPreferredBounds()</TT>. */
    public void setPreferredX(int x) {
        prefx = x;
    }

    /** Returns the x-coordinate for <TT>getPreferredBounds()</TT>. */
    public int getPreferredX() {
        return prefx;
    }

    /**
     * Sets the reversed flag.
     * 
     * @param showReversed <B>true </B>if inputs and outputs are to be drawn in
     *            the reversed position from the type declaration, e.g., when a
     *            <TT>Plug</TT> is drawn; <B>false </B> otherwise.
     */
    public void setReversed(boolean showReversed) {
        reversed = showReversed;
        if (reversed) {
            removeAll();
            initComponents();
        }
    }

    /**
     * Returns <B>true </B> if the element has I/O of the specified type, e.g.,
     * <TT>"EventInputs"</TT>.
     */
    private boolean hasIO(String iotype) {
        NodeList ionodes = getElement().getElementsByTagName(iotype);
        return (ionodes.getLength() < 1) ? false : (ionodes.item(0)
                .getChildNodes().getLength() > 0);
    }
}
