//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Rectangle;

/**
 * A layout manager for a GraphView panel.
 * 
 * @author JHC
 * @version 20050530/JHC
 */
public class GraphLayout implements LayoutManager {

    public GraphLayout() {
        super();
    }

    /**
     * Does nothing since GraphView does not use this method.
     * 
     * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String,
     *      java.awt.Component)
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Does nothing since GraphView does not use this method.
     * 
     * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Returns preferred layout size based on components' getPreferredBounds().
     * 
     * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
     */
    public Dimension preferredLayoutSize(Container parent) {
        Dimension ans = new Dimension();
        Component[] comps = parent.getComponents();
        for (int i = 0; i < comps.length; i++) {
            GraphElement ge = (GraphElement) comps[i];
            Rectangle r = ge.getPreferredBounds();
            if (r != null) {
                ans.width = Math.max(ans.width, r.x + r.width);
                ans.height = Math.max(ans.height, r.y + r.height);
            }
        }
        return ans;
    }

    /**
     * Returns the same answer as preferredLayoutSize().
     * 
     * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
     */
    public Dimension minimumLayoutSize(Container parent) {
        return preferredLayoutSize(parent);
    }

    /**
     * Lays out nodes followed by edges, giving each component its desired
     * bounds from getPreferredBounds().
     * 
     * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
     */
    public void layoutContainer(Container parent) {
        Component[] comps = parent.getComponents();
        for (int i = 0; i < comps.length; i++) {
            GraphElement ge = (GraphElement) comps[i];
            if(ge instanceof GraphEdge) continue;
            Rectangle r = ge.getPreferredBounds();
            if (r != null)
                ge.setBounds(r);
        }
        for (int i = 0; i < comps.length; i++) {
            GraphElement ge = (GraphElement) comps[i];
            if(ge instanceof GraphNode) continue;
            ge.initComponents();
            Rectangle r = ge.getPreferredBounds();
            if (r != null)
                ge.setBounds(r);
        }
    }

}
