//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Area;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.Border;

/**
 * A node of a directed graph.
 * 
 * @author JHC
 * @version 20050630/JHC
 */
public abstract class GraphNode extends GraphElement {
    protected static final Border mainBorder = BorderFactory.createEmptyBorder(
            2, 2, 2, 2);

    protected JLabel mainLabel = new JLabel();

    /** The Area to be used to outline the component. */
    protected Area outline = new Area();

    /** The default background color. */
    public static final Color DEFAULT_BKGD = new Color(224, 224, 255);

    /**
     * Initializes a node of a directed graph corresponding to the given DOM
     * Element. The default node has a bordered label containing the node type
     * with the node name above the border, or just containing the name if there
     * is no type.
     */
    public GraphNode() {
        super();
        setBorder(mainBorder);
        mainLabel.setBorder(GraphElement.emborder);
        mainLabel.setBackground(Color.white);
        mainLabel.setOpaque(true);
        mainLabel.setHorizontalAlignment(JLabel.CENTER);
    }

    public Rectangle getPreferredBounds() {
        int x = Math.round(getScaledAtt("x"));
        int y = Math.round(getScaledAtt("y"));
        Dimension sz = getPreferredSize();
        return new Rectangle(x, y, sz.width, sz.height);
    }

    /**
     * Initialize the sub-components of this node for the given node name and
     * type - called by <TT>setElement()</TT>.
     */
    protected void initComponents() {
        mainLabel.setFont(getFont());
        mainLabel.setText(mainLabelText());
    }

    public Integer getPreferredLayer() {
        return GraphView.NODE_LAYER;
    }

    /**
     * Return the text for the main label. Default is the model's <TT>Name
     * </TT> attribute.
     */
    protected String mainLabelText() {
        return model.getName();
    }

    public void paintBorder(Graphics g) {
        super.paintBorder(g);
        updateShape();
        if(outline.isEmpty()) return;
        Color oldcolor = g.getColor();
        g.setColor(isSelected() ? getSelectionColor() : getDeselectedColor());
        ((Graphics2D) g).fill(outline);
        g.setColor(getBorderColor());
        ((Graphics2D) g).draw(outline);
        g.setColor(oldcolor);
    }

    /** Update the outline shape before drawing the border. Default is to do nothing.*/
    protected void updateShape(){
    }

    /** Returns the default background color for deselected nodes. */
    protected Color getDeselectedColor() {
        return DEFAULT_BKGD;
    }
    /** Returns the border color (default is black). */
    protected Color getBorderColor(){
        return Color.black;
    }
}
