//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.

//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

import javax.swing.JLayeredPane;
import javax.swing.JPanel;

import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.oooneida.wb.graph.model.GraphModel;
import org.oooneida.wb.graph.popup.ContextMenu;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.MutationEvent;

/**
 * A panel for displaying an Element of the DOM Tree of an IEC 61499 XML
 * LibraryElement as a directed graph.
 * 
 * @author JHC, JP
 * 
 * @version 20070530/JP - GraphView extends ViewModel, most of the basic features have been
 * 			taken to ViewModel.java for better extensibility 
 * @version 20070330/JP - Display FBNetwork of Resource element when the Resource is selected on tree
 * @version 20051129/JHC - Inhibited re-drawing in handleEvent when
 *          MutationEvent comes from an Action (this is clumsy and will probably
 *          have to be fixed later).
 * @version 20051109/JHC - All notifications now via DOM Event mechanism.
 * @version 20050421/JHC
 */
public class GraphView extends ViewModel {

	/**
	 * Sets up a GraphView for an IEC 61499 LibraryElement and its graphical
	 * sub-elements contained in the specified DOM Document.
	 */
	public GraphView(Document doc) {
		super();
		((org.w3c.dom.events.EventTarget) doc).addEventListener("ElementSelectionEvent", this, false);
		setLayout(new GraphLayout());
		setOpaque(true);
		setBackground(Color.white);
		setFont(GraphView.DEFAULT_FONT);
		JPanel dragPanel = new JPanel();
		add(dragPanel, DRAG_LAYER);
		addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent evt){
				if(evt.getButton() == MouseEvent.BUTTON3) {
					/**
					 *  Resource is not a FBNetwork but it contains one so
					 *  it is required to pass FBNetwork of Resource as an element
					 *  when Resource is passed in.
					 */
					System.out.println("GraphView."+element.getNodeName());
					Element fbNetworkElem = (Element) (element.getNodeName().equals("Resource")
							? element.getFirstChild() : element);
					String elementName = fbNetworkElem.getNodeName();
					
					ContextMenu contextMenu = new ContextMenu(fbNetworkElem, evt);
					contextMenu.create(elementName);
				}
			}
		});
	}
	
	public void refresh(){
		setElement(element);
	}
	public void handleEvent(Event evt) {
		if (evt instanceof MutationEvent) {
			Element mutel = (Element) ((MutationEvent) evt).getRelatedNode();
			System.out.println("GraphView: "+mutel.getTagName());
			if (!mutel.getTagName().equals("ECAction"))
				setElement(element);
		} else if (evt instanceof ElementSelectionEvent) {
			ElementSelectionEvent selevt = (ElementSelectionEvent) evt;
			switch (selevt.getDetail()) {
			case ElementSelectionEvent.ACTIVATE: {
				graphElementSelected(selevt.getSource());
				return;
			}
			case ElementSelectionEvent.HYPERACTIVATE: {
				setElement(selevt.getSelectedElement());
			}
			}
		}
	}
}