//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * A graphical representation of the link between an I/O name and its type.
 * 
 * @author JHC
 * @version 20051010/JHC
 */
public class IOLink extends GraphEdge {
	protected boolean isEvent = false;

	/**
	 * Initializes a new IOLink.
	 * 
	 * @param src
	 *            The source (left) component.
	 * @param dst
	 *            The destination (right) component.
	 * @param eventFlag
	 *            <TT>true</TT> if this is an Event I/O.
	 */
	public IOLink(Component src, Component dst, boolean eventFlag) {
		super();
		source = src;
		dest = dst;
		isEvent = eventFlag;
	}

	public Point getSourcePoint() {
		return isInput() ? super.getSourcePoint()
				: new Point(source.getX() + source.getWidth(), GraphElement
						.getCenter(dest.getBounds()).y);
	}

	public Point getDestPoint() {
		return !isInput() ? super.getDestPoint() : new Point(dest.getX(),
				GraphElement.getCenter(source.getBounds()).y);
	}
	
	public Component getSource() {
		return source;
	}
	
	public Component getDest() {
		return dest;
	}

	public Component getEndPtComponent(boolean findSource) {
		return null;
	}

	protected boolean isInput() {
		return (dest instanceof FBType);
	}

	/** Returns the index of the receiver's Element in its NodeList of peers. */
	protected int getIndex() {
		int n = 0;
		for (Node prev = getElement(); prev != null; prev = prev
				.getPreviousSibling()) {
			n++;
		}
		return n;
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (!isEvent)
			return;
		//Draw the WITH origin box
		int boxx = getWithBoxX();
		Color oldcolor = g.getColor();
		g.setColor(Color.black);
		int h = getHeight();
		g.fillRect(boxx, 0, h, h);
		g.setColor(oldcolor);
	}

	public void setElement(Element el) {
		model.setElement(el);
	}

	protected int getWithBoxX() {
		int boxx = 10 * getIndex();
		if (isInput())
			boxx = getWidth() - boxx - 5;
		return boxx;
	}
}
