//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

/**
 * A GraphEdge for drawing the left and right boundaries of a ServiceSequence.
 * 
 * @author JHC
 * @version 20051105/JHC
 */
public class ServiceBounds extends GraphEdge {

    /**
     * Constructs a new ServiceBounds between the given ServiceSequnce header
     * and last service primitive.
     *  
     */
    public ServiceBounds(ServiceSequence seqhdr, InputPrimitive lastprim) {
        super();
        source = seqhdr;
        dest = lastprim;
    }

    /** Returns null - Should never be called. */
    protected Component getEndPtComponent(boolean findSource) {
        return null;
    }

    public Point getSourcePoint() {
        return new Point(source.getX()
                + ((Container) source).getComponent(2).getX(), source
                .getHeight());
    }

    public Point getDestPoint() {
        return new Point(getSourcePoint().x, dest.getY() + dest.getHeight());
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.translate(-getX(), -getY());
        Color oldcolor = g.getColor();
        g.setColor(Color.black);
        g.drawLine(pathx[0] + 28, pathy[0], pathx[1] + 28, pathy[1]);
        g.setColor(oldcolor);
        g.translate(getX(), getY());
    }

    /**
     * Returns <TT>false</TT> so that this element is never selected.
     */
    public boolean contains(int x, int y) {
        return false;
    }

    public Rectangle getPreferredBounds() {
        Rectangle r = super.getPreferredBounds();
        r.width += 30;
        return r;
    }

}
