//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

/**
 * A link between ServicePrimitives.
 * 
 * @author JHC
 * @version 20051105/JHC
 */
public class ServiceLink extends GraphEdge {
    protected Rectangle rbuff = new Rectangle();

    /**
     * Constructs a new ServiceLink.
     * 
     * @param src The source Component.
     * @param dst The destination Component.
     */
    public ServiceLink(InputPrimitive src, InputPrimitive dst) {
        super();
        source = src;
        dest = dst;
    }

    /** Returns null - Should never be called. */
    protected Component getEndPtComponent(boolean findSource) {
        return null;
    }

    public Point getSourcePoint() {
        return getLinkPoint(source);
    }

    public Point getDestPoint() {
        return getLinkPoint(dest);
    }

    protected Point getLinkPoint(Component c) {
        c.getBounds(rbuff);
        int x = rbuff.x;
        if (((InputPrimitive) c).isLeftInterface())
            x += rbuff.width;
        return new Point(x, rbuff.y + rbuff.height / 2);
    }

    /**
     * Returns <TT>false</TT> so that this element is never selected.
     */
    public boolean contains(int x, int y) {
        return false;
    }

    public void paintComponent(Graphics g) {
        if (pathx[0] != pathx[1])
            super.paintComponent(g);
        else {
            g.translate(-getX(), -getY());
            Color oldcolor = g.getColor();
            g.setColor(Color.black);
            g.drawOval(pathx[0] - 10, pathy[0], 20, pathy[1] - pathy[0]);
            g.setColor(oldcolor);
            g.translate(getX(), getY());
        }
    }

    public Rectangle getPreferredBounds() {
        Rectangle r = super.getPreferredBounds();
        if (pathx[0] == pathx[1]) {
            r.width = 20;
            if (!((InputPrimitive) source).isLeftInterface())
                r.x -= 20;
        }
        return r;
    }
}
