//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

import javax.swing.JLayeredPane;
import javax.swing.JPanel;

import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.oooneida.wb.graph.model.GraphModel;
import org.oooneida.wb.graph.popup.ContextMenu;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.MutationEvent;

/**
 *
 * An abstract panel for displaying an Element of the DOM Tree of an IEC 61499 XML
 * LibraryElement as a directed graph.
 * 
 * @author JP
 *
 * @version 20070530/JP
 */
public abstract class ViewModel extends JLayeredPane implements EventListener{
	protected GraphElement selectedGraphElement = null;

	protected GraphModel model = null;

	/**
	 * The layer on which edges are plotted (the bottom layer, equivalent to
	 * <TT>JLayeredPane.DEFAULT_LAYER</TT>.
	 */
	public static final Integer EDGE_LAYER = JLayeredPane.DEFAULT_LAYER;

	/**
	 * The layer on which I/O connections are plotted, between EDGE_LAYER and
	 * NODE_LAYER.
	 */
	public static final Integer IO_LAYER = new Integer(
			EDGE_LAYER.intValue() + 1);	

	/** The layer on which nodes are plotted (the top layer). */
	public static final Integer NODE_LAYER = new Integer(
			IO_LAYER.intValue() + 1);	

	protected Element element = null;

	public static final Font DEFAULT_FONT = new Font("SansSerif", Font.PLAIN, 10);

	/**
	 * Sets up a GraphView for an IEC 61499 LibraryElement and its graphical
	 * sub-elements contained in the specified DOM Document.
	 */
	public ViewModel() {
		super();
	}

	protected void graphElementSelected(Object source) {
		if (!(source instanceof GraphElement))
			return;
		if (selectedGraphElement == source)
			return;
		GraphElement ge = (GraphElement) source;
		if (ge.getParent() != this)
			return;
		if (selectedGraphElement != null)
			selectedGraphElement.setSelected(false);
		selectedGraphElement = ge;
		ge.setSelected(true);
		repaint(ge.getPreferredBounds());
		return;
	}

	public void setElement(Element el) {
		System.out.println(el.getNodeName()+" "+el.getChildNodes().getLength());
		if (element != null)
			((org.w3c.dom.events.EventTarget) element).removeEventListener(
					"MutationEvent", this, false);
		element = el;
		((org.w3c.dom.events.EventTarget) el).addEventListener("MutationEvent", this, false);
		removeAll();
		model = GraphModel.graphModelForElement(element);
		if (model != null) {
			for (Enumeration els = model.getGraph().elements(); els
					.hasMoreElements();) {
				GraphElement ge = (GraphElement) els.nextElement();
				add(ge, ge.getPreferredLayer());
				ge.addVisibleParameters(this);
			}
		}
		validate();
		doLayout();
		repaint();
	}
	
	public void handleEvent(Event evt) {
	}
	
	public GraphModel getModel() {
		return model;
	}
	public GraphElement getSelectedGraphElement(){
		return selectedGraphElement;
	}
}
