//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * A graphical representation of the vertical line between an Event and its WITH
 * associations.
 * 
 * @author JHC
 * @version 20051012/JHC
 */
public class WithLink extends GraphEdge {
	/** The names of the WITH elements of this link's Event. */
	Vector withnames = new Vector();

	/**
	 * Initializes a new WithLink.
	 * 
	 * @param src
	 *            The source (IOLink) component.
	 * @param dst
	 *            The destination (FBType) component.
	 */
	public WithLink(IOLink src, FBType dst) {
		super();
		setElement(src.getElement());
		source = src;
		dest = dst;
	}

	public Point getSourcePoint() {
		Point p = ((IOLink) source).getSourcePoint();
		p.x += ((IOLink) source).getWithBoxX();
		return p;
	}

	public Point getDestPoint() {
		Point p = getSourcePoint();
		NodeList withs = getElement().getChildNodes();
		withnames.removeAllElements();
		for (int i = 0; i < withs.getLength(); i++)
			withnames.addElement(((Element) withs.item(i)).getAttribute("Var"));
		Component[] comps = getParent().getComponents();
		for (int i = 0; i < comps.length; i++) {
			Component comp = comps[i];
			if (!(comp instanceof IOLink))
				continue;
			IOLink lnk = (IOLink) comp;
			if (withnames.contains(lnk.getElement().getAttribute("Name")))
				p.y = Math.max(p.y, lnk.getSourcePoint().y);
		}
		return p;
	}

	protected Component getEndPtComponent(boolean findSource) {
		return null;
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Point p = getLocation();
		g.translate(-p.x, -p.y);
		Color oldcolor = g.getColor();
		g.setColor(Color.BLACK);
		int boxx = getSourcePoint().x - 2;
		Component[] comps = getParent().getComponents();
		for (int i = 0; i < comps.length; i++) {
			Component comp = comps[i];
			if (!(comp instanceof IOLink))
				continue;
			IOLink lnk = (IOLink) comp;
			if (withnames.contains(lnk.getElement().getAttribute("Name"))) {
				int h = lnk.getHeight();
				g.fillRect(boxx, lnk.getY(), h, h);
			}
		}
		g.setColor(oldcolor);
		g.translate(p.x, p.y);
	}
}
