//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph.model;

import java.util.Vector;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.oooneida.wb.dom.ElementModel;
import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.DocumentEvent;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * A TableModel for the ECActions of an ECStep (used by
 * org.oooneida.wb.graph.ECAction).
 * 
 * @author JHC
 * @version 20051129/JHC - Subclassed from ElementModel instead of
 *          AbstractTableModel; implemented TableModel; added
 *          MutationEvent notification in setValueAt().
 * @version 20051028/JHC
 */
public class ActionsModel extends ElementModel implements TableModel {
	private NodeList actions;

	private MutationEvent mutev = null;

	private ElementSelectionEvent selevt = new ElementSelectionEvent();

	private Vector listeners = new Vector(4);
	
	/** Constructs a table model for the ECActions of the given ECState Element. */
	public ActionsModel(Element el) {
		super(el);
		actions = el.getChildNodes();
		mutev = (MutationEvent) ((DocumentEvent) el.getOwnerDocument())
				.createEvent("MutationEvent");
		mutev.initEvent("MutationEvent", true, false);
		selevt.initSelectionEvent(el,this,ElementSelectionEvent.ACTIVATE);
	}

	public int getRowCount() {
		return actions.getLength();
	}

	public int getColumnCount() {
		return 2;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return ((Element) actions.item(rowIndex)).getAttribute("Algorithm");
		case 1:
			return ((Element) actions.item(rowIndex)).getAttribute("Output");
		}
		return null;
	}

	public boolean isCellEditable(int row, int column) {
		return true;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		String newval = aValue.toString();
		if(newval.equals(getValueAt(rowIndex,columnIndex).toString())) return;
		Element action = (Element)actions.item(rowIndex);
		action.setAttribute((columnIndex == 0) ? "Algorithm" : "Output", newval);
		((org.w3c.dom.events.EventTarget) action).dispatchEvent(mutev);
	}

	public String getColumnName(int columnIndex) {
		switch (columnIndex) {
		case 0:
			return "Attribute";
		case 1:
			return "Value";
		}
		return "";
	}

	public Class getColumnClass(int columnIndex) {
		return String.class;
	}

	public void addTableModelListener(TableModelListener l) {
		if (!listeners.contains(l))
			listeners.addElement(l);

	}

	public void removeTableModelListener(TableModelListener l) {
		listeners.removeElement(l);
	}
}