//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.graph.model;

import java.util.Vector;

import org.oooneida.wb.graph.Event;
import org.oooneida.wb.graph.GraphElement;
import org.oooneida.wb.graph.IOLink;
import org.oooneida.wb.graph.WithLink;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * A GraphModel for the interface of an FBType element.
 * 
 * @author JHC
 * @version 20050926/JHC
 */
public class FBType extends GraphModel {

	public FBType() {
		super();
	}

	public FBType(Element el) {
		super(el);
	}

	/** The vertical location to install the next I/O element. */
	private int y = 0;

	/** The vertical spacing of I/O elements. */
	private int dy = 0;

	/**
	 * The horizontal location for <TT>getPreferredBounds()</TT> of the FB
	 * type block.
	 */
	private int fbx = 0;

	/** The gap between the edge of the block and an I/O element. */
	private int gap = 0;

	/** The FBType outline component. */
	private org.oooneida.wb.graph.FBType fbt = null;

	/**
	 * Returns the GraphElements to draw the FBType's outline and its I/O
	 * declarations.
	 */
	public Vector getGraph() {
		fbx = 0;
		Vector ans = new Vector();
		fbt = (org.oooneida.wb.graph.FBType) GraphElement.forElement(getElement());
		ans.add(fbt);
		int y0 = fbt.getPreferredBounds().y + 2;
		dy = fbt.getFontMetrics(fbt.getFont()).getHeight() + 2;
		NodeList intfclst = getElement().getElementsByTagName("InterfaceList");
		if (intfclst.getLength() <= 0)
			return ans;
		Element intfcs = (Element) intfclst.item(0);
		y = y0;
		addInterfaceElements(intfcs, "EventInputs", ans, true, true, 0);
		int y1 = y;
		NodeList lst = intfcs.getElementsByTagName("EventOutputs");
		if (lst.getLength() >0){
			int n = lst.item(0).getChildNodes().getLength();
			y1 = Math.max(y1,y0+dy*n);
		}
		y1 += dy * 2 - 2;
		y = y1;
		addInterfaceElements(intfcs, "InputVars", ans, true, false, 0);
		addInterfaceElements(intfcs, "Sockets", ans, true, false, 0);
		fbt.setPreferredX(fbx);
		y = y0;
		int x0 = fbx + fbt.getPreferredSize().width;
		addInterfaceElements(intfcs, "EventOutputs", ans, false, true, x0);
		y = y1;
		addInterfaceElements(intfcs, "OutputVars", ans, false, false, x0);
		addInterfaceElements(intfcs, "Plugs", ans, false, false, x0);
		return ans;
	}

	/**
	 * Adds a set of interface elements to the graph being built in <TT>
	 * getGraph()</TT>.
	 * 
	 * @param intfcs
	 *            The <TT>InterfaceList</TT> element from which to retrieve
	 *            the interface elements.
	 * @param tag
	 *            The name of the subset of interface elements to be added,
	 *            e.g., <TT>InputVars</TT>.
	 * @param graph
	 *            The Vector of GraphElements.
	 * @param isInputs
	 *            <TT>true</TT> if the elements to be added are inputs.
	 * @param isEvents
	 *            <TT>true</TT> if the elements to be added are events.
	 * @param x0
	 *            The horizontal location of elements to be added.
	 */
	protected void addInterfaceElements(Element intfcs, String tag,
			Vector graph, boolean isInputs, boolean isEvents, int x0) {
		NodeList lst = intfcs.getElementsByTagName(tag);
		if (lst.getLength() < 1)
			return;
		Event evt = null;
		NodeList evts = lst.item(0).getChildNodes();
		int n = evts.getLength();
		if (isEvents)
			gap = 10 * n + 10;
		if (!isInputs)
			x0 += gap;
		for (int i = 0; i < n; i++) {
			evt = (Event) GraphElement.forElement((Element) evts.item(i));
			evt.setPreferredLocation(x0, y);
			graph.add(evt);
			y += dy;
			if (isInputs)
				fbx = Math.max(fbx, evt.getPreferredSize().width + gap);
			IOLink lnk = isInputs ? new IOLink(evt, fbt, isEvents)
					: new IOLink(fbt, evt, isEvents);
			lnk.setElement(evt.getElement());
			graph.add(lnk);
			if (isEvents)
				graph.add(new WithLink(lnk, fbt));
		}
	}
}
