//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph.popup;

import static javax.swing.GroupLayout.Alignment.BASELINE;
import static javax.swing.GroupLayout.Alignment.CENTER;
import static javax.swing.GroupLayout.Alignment.LEADING;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import org.oooneida.wb.dom.DOMTextModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Dialog which extends DialogModel for adding, editing and 
 * deleting components of FBType GraphModel.
 * 
 * @author JP
 * @version 20070228/JP
 */
public class FBTypeDialog extends DialogModel{

	private JLabel nameLabel = new JLabel("Name :");
	private JLabel typeLabel = new JLabel("Type :");
	private JLabel commentLabel = new JLabel("Comment :");
	private JLabel arraySizeLabel = new JLabel("Array Size :");
	private JLabel initialValueLabel = new JLabel("Initial Value :");

	private JTextField nameField = new JTextField();
	private JTextField commentField = new JTextField();
	private JTextField arraySizeField = new JTextField();
	private JTextField initialValueField = new JTextField();

	private JPanel variableDetailPane;
	private JPanel variableListPane;

	private JTable variableTable = null;

	private String[] typeOptions = { "ANY", "ARRAY", "BOOL", "BYTE", "COLOR",
			"DATE_AND_TIME", "DINT", "DWORD", "INT", "LINT", "LREAL",
			"LWORD", "MATRIX", "REAL", "SINT", "STRING", "TIME", "UDINT",
			"UINT", "USINT", "WORD", "WSTRING" };
	private JComboBox typeBox = new JComboBox(typeOptions);

	private JButton acceptButton;
	private JButton cancelButton = new JButton("Cancel");
	private JButton helpButton = new JButton("Help");

	protected enum EventType {
		Input, Output;
	};
	protected enum VariableType {
		Input, Output, Internal;
	}

	private EventType eventType;
	private VariableType variableType;

	/**
	 * element is GraphModel element when FBTypeDialog is created for
	 * the purpose of <B>adding</B> a new event or variable,
	 * otherwise (if its purpose is for <b>editing</b> or <b>deleting</b>), 
	 * element is the selected GraphElement element.
	 */
	public FBTypeDialog(Element element, MouseEvent mouseEvt) {
		super(element, mouseEvt);
	}

	@Override
	public void create(String command) {
		if(command.equals("Add Input Event")){
			createDialogForEvent(EventType.Input, null);
		} else if(command.equals("Add Output Event")){
			createDialogForEvent(EventType.Output, null);
		} else if(command.equals("Add Input Variable")){
			createDialogForVariable(VariableType.Input, null);
		} else if(command.equals("Add Output Variable")){
			createDialogForVariable(VariableType.Output, null);
		} else if(command.equals("Add Internal Variable")){
			createDialogForVariable(VariableType.Internal, null);
		} 

		else if(command.equals("Edit Event")){
			Element eventElement = rearrangeElement();
			createDialogForEvent(null, eventElement);
		} else if(command.equals("Edit Variable")){
			Element variableElement = rearrangeElement();
			createDialogForVariable(null, variableElement);
		} 

		else if(command.equals("Delete Event") || command.equals("Delete Variable")){
			createRemoveDialog();
		}
		setVisible(true);
	}




	private boolean isForAddingEvent = false;
	private Element eventElement = null;
	private void createDialogForEvent(EventType et, Element eventElem){
		isForAddingEvent = (eventElem == null) ? true : false;
			
		setTitle(isForAddingEvent ? "Add "+eventType+" Event" : "Edit Event");
		setDialogSize(400, 260);

		eventElement = eventElem;
		eventType = et;
		
		variableListPane = getVariableListPane();
		acceptButton = new JButton(isForAddingEvent ? "Add" : "Accept");

		acceptButton.addActionListener(new AddEventButtonListener());
		cancelButton.addActionListener(new CancelButtonActionListener());
		helpButton.addActionListener(null);
		setLayoutForEvent();
	}

	private void setLayoutForEvent() {
		layout.setHorizontalGroup(layout.createParallelGroup(CENTER)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(LEADING)
								.addComponent(nameLabel)
								.addComponent(commentLabel))
								.addGroup(layout.createParallelGroup(LEADING)
										.addComponent(nameField)
										.addComponent(commentField)))
										.addGroup(layout.createSequentialGroup()
												.addGap(10)
												.addComponent(variableListPane)
												.addGap(10))
												.addGroup(layout.createSequentialGroup()
														.addComponent(acceptButton)
														.addComponent(cancelButton)
														.addComponent(helpButton)));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(BASELINE)
						.addComponent(nameLabel)
						.addComponent(nameField))
						.addGroup(layout.createParallelGroup(BASELINE)
								.addComponent(commentLabel)
								.addComponent(commentField))
								.addComponent(variableListPane)
								.addGroup(layout.createParallelGroup(BASELINE)
										.addComponent(acceptButton)
										.addComponent(cancelButton)
										.addComponent(helpButton)));

		if(!isForAddingEvent){
			nameField.setText(eventElement.getAttribute("Name"));
			commentField.setText(eventElement.getAttribute("Comment"));
		}
	}


	private JPanel getVariableListPane(){
		JPanel contentPane = new JPanel();
		GroupLayout cpLayout = new GroupLayout(contentPane);
		contentPane.setLayout(cpLayout);

		TitledBorder border = new TitledBorder("Associated Variables");
		border.setTitleFont(border.getTitleFont().deriveFont(new Float(10)));
		contentPane.setBorder(border);

		/**
		 * Retrieves list of all the input or output variables that belongs to this FunctionBlock.</br>
		 * Its class variable <B>Vector associatedVariableList</B> consists of
		 * Vector variables that has name of a variable as a first element and a Boolean variable
		 * as a second element to indicate whether the given variable is associated with the event.
		 *  
		 * @author Jason
		 */
		class Variables{
			private Vector<Vector> associatedVariableList = new Vector<Vector>();

			private Variables(Element eventElement){
				NodeList eventInputNL = document.getElementsByTagName("EventInputs");
				if(eventInputNL.getLength() != 0){
					NodeList inputEventNodes = eventInputNL.item(0).getChildNodes();
					boolean isInputEvent = false;
					for(int i = 0; i < inputEventNodes.getLength(); i++){
						if(inputEventNodes.item(i).equals(eventElement)){
							isInputEvent = true;
							break;
						}
					}
					updateAssociatedVariableList(isInputEvent);
				}
			}
			private Variables(EventType et){
				boolean isInputEvent = false;
				if(et == EventType.Input)
					isInputEvent = true;
				else if(et == EventType.Output)
					isInputEvent = false;
				updateAssociatedVariableList(isInputEvent);
			}

			private void updateAssociatedVariableList(boolean isInputEvent) {
				NodeList variableNodes;
				if(isInputEvent){
					try{
						variableNodes = document.getElementsByTagName("InputVars").item(0).getChildNodes();
					} catch (NullPointerException e){
						variableNodes = null;
						associatedVariableList = new Vector<Vector>();
						return;
					}
				}
				else{
					try{
						variableNodes = document.getElementsByTagName("OutputVars").item(0).getChildNodes();
					} catch (NullPointerException e){
						variableNodes = null;
						associatedVariableList = new Vector<Vector>();
						return;
					}
				}

				for(int i = 0; i < variableNodes.getLength(); i++){
					Element variableElem = (Element) variableNodes.item(i);
					Vector variable = new Vector(2);
					variable.add(0, variableElem.getAttribute("Name"));
					associatedVariableList.add(variable);
				}
				try{
					NodeList associatedVarList = eventElement.getChildNodes();
					for(int i = 0; i < associatedVariableList.size(); i++){
						String variableName = associatedVariableList.get(i).get(0).toString();
						for(int j = 0; j < associatedVarList.getLength(); j++){
							String associatedVarName = ((Element) associatedVarList.item(j)).getAttribute("Var");
							if(variableName.equals(associatedVarName))
								associatedVariableList.get(i).add(1, new Boolean(true));
							else {
								if(associatedVariableList.get(i).size() == 1){
									associatedVariableList.get(i).add(1, new Boolean(false));
								}
							}
						}
					}
				} catch (NullPointerException e){
					for(int i = 0; i < associatedVariableList.size(); i++) {
						associatedVariableList.get(i).add(1, new Boolean(false));
					}
				}

			}

			private Vector<Vector> getAssociatedVarList(){
				return associatedVariableList;
			}
		}
		Vector<Vector> variableList;
		Variables variables;
		if(isForAddingEvent)
			variables = new Variables(eventType);

		else 
			variables = new Variables(eventElement);
		variableList = variables.getAssociatedVarList();
		Vector<String> columnHeading = new Vector<String>(2);
		columnHeading.add("Variable");
		columnHeading.add("Assocated");

		variableTable = new JTable(variableList.size(), 2);

		DefaultTableModel vtTableModel = (DefaultTableModel)variableTable.getModel();
		vtTableModel.setDataVector(variableList, columnHeading);

		/**
		 * Set CellRenderer and CellEditor if there is any element in the table.
		 * Otherwise, skip this setting.
		 */
		if(variableList.size() > 0){
			JCheckBox checkBox = new JCheckBox();
			DefaultCellEditor cellEditor = new DefaultCellEditor(checkBox);
			checkBox.setHorizontalAlignment(SwingConstants.CENTER);
			variableTable.getColumnModel().getColumn(1).setCellEditor(cellEditor);
			TableCellRenderer booleanCellRenderer = 
				(TableCellRenderer) variableTable.getDefaultRenderer(vtTableModel
						.getValueAt(0, 1).getClass())
						.getTableCellRendererComponent
						(variableTable, variableList.get(0).get(1),false, false, 0, 1);
			variableTable.getColumnModel().getColumn(1).setCellRenderer(booleanCellRenderer);
		}

		variableTable.setGridColor(Color.gray);
		float fontSizeTen = 10;
		variableTable.setFont(getFont().deriveFont(fontSizeTen));
		JScrollPane variableTablePane = new JScrollPane(variableTable);
		variableTablePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

		cpLayout.setHorizontalGroup(cpLayout.createSequentialGroup()
				.addGap(7)
				.addComponent(variableTablePane)
				.addGap(5));
		cpLayout.setVerticalGroup(cpLayout.createSequentialGroup()
				.addComponent(variableTablePane)
				.addGap(5));
		Dimension size = new Dimension(350,112);
		contentPane.setMinimumSize(size);
		contentPane.setMaximumSize(size);
		return contentPane;
	}

	private boolean isForAddingVar = false;
	private Element varElement = null;
	private void createDialogForVariable(VariableType vt, Element varElem){
		setTitle(isForAddingVar ? "Add "+variableType+" Variable" : "Edit Variable");
		setDialogSize(350, 210);

		varElement = varElem;
		variableType = vt;
		if(varElem == null)
			isForAddingVar  = true;
		else
			isForAddingVar = false;

		variableDetailPane = createVariableDetailPane();
		acceptButton = new JButton(isForAddingVar ? "Add" : "Accept");
		acceptButton.addActionListener(new AddVariableButtonListener());
		cancelButton.addActionListener(new CancelButtonActionListener());

		setLayoutForVariable();
	}

	private void setLayoutForVariable() {
		layout.setHorizontalGroup(layout.createParallelGroup(CENTER)
				.addGroup(layout.createSequentialGroup()
						.addComponent(nameLabel)
						.addComponent(nameField))
						.addComponent(variableDetailPane)
						.addGroup(layout.createSequentialGroup()
								.addComponent(commentLabel)
								.addComponent(commentField))
								.addGroup(layout.createSequentialGroup()
										.addComponent(acceptButton)
										.addComponent(cancelButton)
										.addComponent(helpButton)));

		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(nameLabel)
						.addComponent(nameField))
						.addComponent(variableDetailPane)
						.addGroup(layout.createParallelGroup(CENTER)
								.addComponent(commentLabel)
								.addComponent(commentField))
								.addGroup(layout.createParallelGroup(CENTER)
										.addComponent(acceptButton)
										.addComponent(cancelButton)
										.addComponent(helpButton)));

		if(!isForAddingVar){
			nameField.setText(varElement.getAttribute("Name"));
			commentField.setText(varElement.getAttribute("Comment"));
			typeBox.setSelectedItem(varElement.getAttribute("Type"));
			arraySizeField.setText(varElement.getAttribute("ArraySize"));
			initialValueField.setText(varElement.getAttribute("InitialValue"));
		}
	}
	private JPanel createVariableDetailPane(){
		JPanel contentPane = new JPanel();
		GroupLayout layout = new GroupLayout(contentPane);
		layout.setAutoCreateGaps(true);
		contentPane.setLayout(layout);

		Font sizeTenFont = getFont().deriveFont(new Float(10));
		TitledBorder border = new TitledBorder("Variable Details");
		border.setTitleFont(sizeTenFont);

		contentPane.setBorder(border);

		arraySizeLabel.setFont(sizeTenFont);
		typeLabel.setFont(sizeTenFont);
		initialValueLabel.setFont(sizeTenFont);

		arraySizeField.setFont(sizeTenFont);
		typeBox.setFont(sizeTenFont);
		initialValueField.setFont(sizeTenFont);

		typeBox.setEditable(false);

		arraySizeLabel.setMaximumSize(new Dimension(arraySizeLabel.getWidth()+10, arraySizeLabel.getHeight()));
		arraySizeField.setMinimumSize(new Dimension(20, arraySizeField.getHeight()));

		layout.setHorizontalGroup(layout.createSequentialGroup()
				.addGap(10)
				.addGroup(layout.createParallelGroup(LEADING)
						.addComponent(typeLabel)
						.addComponent(initialValueLabel))
						.addGroup(layout.createParallelGroup(LEADING)
								.addGroup(layout.createSequentialGroup()
										.addComponent(typeBox)
										.addComponent(arraySizeLabel)
										.addComponent(arraySizeField))
										.addComponent(initialValueField))
										.addGap(10));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(typeLabel)
						.addComponent(typeBox)
						.addComponent(arraySizeLabel)
						.addComponent(arraySizeField))
						.addGroup(layout.createParallelGroup(CENTER)
								.addComponent(initialValueLabel)
								.addComponent(initialValueField))
								.addGap(7));

		return contentPane;
	}


	private boolean checkName(){
		String newName = nameField.getText().toUpperCase();		
		if(newName.replaceAll(" ", "").length() == 0){
			JOptionPane.showMessageDialog(me, "Name field is empty",
					"Warning", JOptionPane.WARNING_MESSAGE);
			return false;
		}

		if(newName.replaceAll(" ", "").length() == 0){
			JOptionPane.showMessageDialog(me, "Name field is empty",
					"Warning", JOptionPane.WARNING_MESSAGE);
			return false;
		}

		String[] list = {"EventInputs", "EventOutputs", "InputVars", "OutputVars", "InternalVars"};

		for(String listname : list){
			if(doesListContainName(listname, newName))
				return false;
		}
		return true;
	}



	private boolean doesListContainName(String listName, String newName){
		NodeList list = null;
		try{
			list = document.getElementsByTagName(listName).item(0).getChildNodes();
		} catch (NullPointerException e) {
			return false;
		}

		for(int i = 0; i < list.getLength(); i++){
			if(((Element)list.item(i)).getAttribute("Name").equals(newName)){
				JOptionPane.showMessageDialog(me, "There is an Interface Component with the same name",
						"Event Warning", JOptionPane.WARNING_MESSAGE);
				return true;
			}
		}
		return false;
	}

	class AddEventButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent evt) {
			String newName = nameField.getText().toUpperCase();

			//Processes 'adding' new Event element into the document
			if(isForAddingEvent){
				if(!checkName())
					return;
				eventElement = document.createElement("Event");
				eventElement.setAttribute("Name", newName);
				if(commentField.getText().length() > 0)
					eventElement.setAttribute("Comment", commentField.getText());

				if(eventType == EventType.Input){
					Node eventInputs = document.getElementsByTagName("EventInputs").item(0);

					// Check if there is <EventInputs> in the document.  If not, create one.
					if(eventInputs == null){
						eventInputs = document.createElement("EventInputs");
						Node interfaceList = document.getElementsByTagName("InterfaceList").item(0);
						if(interfaceList.getChildNodes().getLength() == 0){
							interfaceList.appendChild(eventInputs);
						}
						else {
							interfaceList.insertBefore(eventInputs, interfaceList.getFirstChild());
						}
					}
					eventInputs.appendChild(eventElement);
				} else if(eventType == EventType.Output){
					Node eventOutputs = document.getElementsByTagName("EventOutputs").item(0);

					// Check if there is <EventOutputs> in the document.  If not, create one.
					if(eventOutputs == null){
						eventOutputs = document.createElement("EventOutputs");
						Node interfaceList = document.getElementsByTagName("InterfaceList").item(0);
						if(interfaceList.getChildNodes().getLength() == 0){
							interfaceList.appendChild(eventOutputs);
						}
						else {
							// If there is no <EventInputs> in <InterfaceList>, <EventOutputs> is the first element
							if(document.getElementsByTagName("EventInputs").item(0) == null){
								interfaceList.insertBefore(eventOutputs, interfaceList.getFirstChild());
							}
							// If there is <EventInputs> in <InterfaceList>, <EventOutputs> is the second element
							else {
								Node eventInputs = 
									interfaceList.replaceChild(eventOutputs, interfaceList.getFirstChild());
								interfaceList.insertBefore(eventInputs, interfaceList.getFirstChild());
							}
						}
					}
					eventOutputs.appendChild(eventElement);
				}
			}

			// Processes 'modifying' the selected Event element.
			else {
				String oldName = eventElement.getAttribute("Name");
				if(!nameField.getText().equals(oldName)){
					if(!checkName())
						return;
					else
						eventElement.setAttribute("Name", newName);
				}
				if(commentField.getText().replaceAll(" ", "").length() != 0)
					eventElement.setAttribute("Comment", commentField.getText());
				else
					if(eventElement.hasAttribute("Comment"))
						eventElement.removeAttribute("Comment");
			}

			// Assign association with the Event and the selected variables.
			while(eventElement.getChildNodes().getLength() != 0)
				eventElement.removeChild(eventElement.getFirstChild());
			
			TableModel model = variableTable.getModel();
			for(int i = 0; i < model.getRowCount(); i++){
				if(model.getValueAt(i, 1).toString().equals("true")){
					Element newElem = document.createElement("With");
					newElem.setAttribute("Var", model.getValueAt(i, 0).toString());
					eventElement.appendChild(newElem);
				}
			}

			// Repaint GraphView
			getGraphView().setElement(getElement());
			dispose();
		}
	}

	class AddVariableButtonListener implements ActionListener{
		
		private void insertAfter(Node newElem, Node selectedElem){
			getElement().replaceChild(newElem, selectedElem);
			getElement().insertBefore(selectedElem, newElem);
		}
		
		private void insertVarForDRType(Node node){
			String nodeName = node.getNodeName();
			if(nodeName.equals("VarDeclaration") || nodeName.equals("CompilerInfo")||
					nodeName.equals("VersionInfo") || nodeName.equals("Identification"))
				insertAfter(varElement, node);
			else 
				insertVarForDRType(node.getPreviousSibling());
		}
		
		public void actionPerformed(ActionEvent evt) {
			String newName = nameField.getText().toUpperCase();
			if(isForAddingVar){
				if(!checkName())
					return;
				// Check the value of ArraySize
				// Only positive values are allowed for ArraySize
				String arraySizeString = arraySizeField.getText();
				int arraySize = 0;
				try{
					if(arraySizeString.length() != 0){
						arraySize = Integer.parseInt(arraySizeString);
						if(arraySize <= 0){
							JOptionPane.showMessageDialog(me, "Please re-enter ArraySize",
									"Invalid ArraySize", JOptionPane.WARNING_MESSAGE);
							return;
						}
					}
				}
				catch(NumberFormatException e){
					JOptionPane.showMessageDialog(me, "Please re-enter ArraySize",
							"Invalid ArraySize", JOptionPane.WARNING_MESSAGE);
					return;
				}

				// Set attributes of new Variable
				varElement = document.createElement("VarDeclaration");
				varElement.setAttribute("Name", newName);
				if(arraySize != 0)
					varElement.setAttribute("ArraySize", ""+arraySize);
				
				varElement.setAttribute("Type", typeOptions[typeBox.getSelectedIndex()]);
				
				if(initialValueField.getText().replaceAll(" ", "").length() > 0)
					varElement.setAttribute("InitialValue", initialValueField.getText());
				if(commentField.getText().replaceAll(" ", "").length() > 0)
					varElement.setAttribute("Comment", commentField.getText().toUpperCase());
				
				boolean isDRType = (getElement().getNodeName().equals("ResourceType") ||
						getElement().getNodeName().equals("DeviceType"));
				
				if(variableType == VariableType.Input && !isDRType){
					Node inputVars = document.getElementsByTagName("InputVars").item(0);

					// Check if there is <inputVars> in the document.  If not, create one.
					if(inputVars == null){
						inputVars = document.createElement("InputVars");
						Node interfaceList = document.getElementsByTagName("InterfaceList").item(0);
						if(!interfaceList.hasChildNodes())
							interfaceList.appendChild(inputVars);
						else {
							Node eventInputs = document.getElementsByTagName("EventInputs").item(0);
							Node eventOutputs = document.getElementsByTagName("EventOutputs").item(0);

							// If there is <EventOutputs> in <InterfaceList>, <InputVars> is the third element
							if(eventOutputs != null)
								insertAfter(inputVars, eventOutputs);

							// If there is <EventInputs> and there is no <EventOutputs> in <InterfaceList>, 
							// EventOutputs is the second element
							else if (eventInputs != null)
								insertAfter(inputVars, eventInputs);
						}
					}
					inputVars.appendChild(varElement);
				} 
				else if(variableType == VariableType.Input&& isDRType){
					insertVarForDRType(getElement().getLastChild());
				}
				else if(variableType == VariableType.Output){
					Node outputVars = document.getElementsByTagName("OutputVars").item(0);

					if(outputVars == null) {
						outputVars = document.createElement("OutputVars");
						Node interfaceList = document.getElementsByTagName("InterfaceList").item(0);

						if(!interfaceList.hasChildNodes())
							interfaceList.appendChild(outputVars);
						else {
							Node inputVars = document.getElementsByTagName("InputVars").item(0);
							Node eventOutputs = document.getElementsByTagName("EventOutputs").item(0);
							Node eventInputs = document.getElementsByTagName("EventInputs").item(0);

							// Append <OutputVars> at appropriate position.
							if(inputVars != null)
								insertAfter(outputVars, inputVars);
							else if(eventOutputs != null)
								insertAfter(outputVars, eventOutputs);
							else if(eventInputs != null)
								insertAfter(outputVars, eventInputs);
							else
								insertAfter(outputVars, interfaceList.getFirstChild());						}
					}
					outputVars.appendChild(varElement);
				} 
				else if(variableType == VariableType.Internal){
					Node internalVars = document.getElementsByTagName("InternalVars").item(0);
					if(internalVars == null){
						Node basicFB = document.getElementsByTagName("BasicFB").item(0);
						internalVars = document.createElement("InternalVars");
						
						document.insertBefore(internalVars, basicFB);
					}
					internalVars.appendChild(varElement);
				}
			}
			
			// Process editing the variable element of the interface
			else {
				String oldName = varElement.getAttribute("Name");

				if(!oldName.equals(newName)){
					String[] list = {"EventInputs", "EventOutputs", "InputVars", "OutputVars"};
					for(String listName : list)
						if(doesListContainName(listName, newName))
							return;
				}
				
				varElement.setAttribute("Name", nameField.getText().toUpperCase());
				varElement.setAttribute("Type", typeOptions[typeBox.getSelectedIndex()]);
				
				if(commentField.getText().replaceAll(" ", "").length() > 0)
					varElement.setAttribute("Comment", commentField.getText().toUpperCase());
				else
					varElement.removeAttribute("Comment");

				if(arraySizeField.getText().replaceAll(" ", "").length() > 0)
					varElement.setAttribute("ArraySize", commentField.getText());
				else
					varElement.removeAttribute("ArraySize");

				if(initialValueField.getText().replaceAll(" ", "").length() > 0)
					varElement.setAttribute("InitialValue", initialValueField.getText());
				else
					varElement.removeAttribute("InitiahValue");
				
				// Update the association link with Event and Variable
				NodeList events = document.getElementsByTagName("Event");
				for(int i = 0; i < events.getLength(); i++){
					NodeList withVars = events.item(i).getChildNodes();
					for(int j = 0; j < withVars.getLength(); j++){
						Element var = ((Element)withVars.item(j));
						if(var.getAttribute("Var").equals(oldName))
							var.setAttribute("Var", newName);
					}
				}
			}
			getGraphView().setElement(getElement());
			dispose();
		}
	}
}