//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.graph.popup;

import static javax.swing.GroupLayout.Alignment.CENTER;
import static javax.swing.GroupLayout.Alignment.LEADING;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.w3c.dom.Element;

/**
 * Dialog which extends DialogModel for adding, editing and 
 * deleting components of Parameter GraphModel.
 * 
 * @author JP
 * @version 20070530/JP
 */
public class ParameterDialog extends DialogModel{

	private JLabel nameLabel = new JLabel("Name :");
	private JLabel valueLabel = new JLabel("Value :");
	private JLabel commentLabel = new JLabel("Comment :");
	
	private JTextField nameField = new JTextField();
	private JTextField valueField = new JTextField();
	private JTextField commentField = new JTextField();
	
	private JButton acceptButton = new JButton("Accept");
	private JButton cancelButton = new JButton("Cancel");
	
	public ParameterDialog(Element element, MouseEvent mouseEvt) {
		super(element, mouseEvt);
		setDialogSize(340, 155);
	}

	@Override
	public void create(String command) {
		if(command.startsWith("Add Parameter."))
			createAddDialog(command.replaceFirst("Add Parameter.", ""));
		else if(command.startsWith("Edit Parameter"))
			createEditDialog();
		/*
		else if(command.equals("Delete Parameter")){
			createRemoveDialog();
		}*/
		
		setVisible(true);
	}
	
	
	private void createAddDialog(String name) {
		/** Note that getElement() returns FB(inc Device, System etc) not Parameter */
		setTitle("Add Parameter");
		
		nameField.setText(name);
		nameField.setEnabled(false);
		
		acceptButton.addActionListener(new AddButtonListener());
		cancelButton.addActionListener(new CancelButtonActionListener());
		
		drawGUI();
	}
	
	private void createEditDialog(){
		setTitle("Add Parameter");
		
		String name = getElement().getAttribute("Name");
		nameField.setText(name);
		nameField.setEnabled(false);
		
		String value = getElement().getAttribute("Value");
		valueField.setText(value);
		valueField.setEnabled(true);
		
		String comment = getElement().getAttribute("Comment");
		commentField.setText(comment);
		commentField.setEnabled(true);
		
		acceptButton.addActionListener(new EditButtonListener());
		cancelButton.addActionListener(new CancelButtonActionListener());
		
		drawGUI();
	}
	
	private void drawGUI(){
		layout.setHorizontalGroup(layout.createParallelGroup(CENTER)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(LEADING)
								.addComponent(nameLabel)
								.addComponent(valueLabel)
								.addComponent(commentLabel))
						.addGroup(layout.createParallelGroup(LEADING)
								.addComponent(nameField)
								.addComponent(valueField)
								.addComponent(commentField)))
				.addGroup(layout.createSequentialGroup()
						.addComponent(acceptButton)
						.addComponent(cancelButton)));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(nameLabel)
						.addComponent(nameField))
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(valueLabel)
						.addComponent(valueField))
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(commentLabel)
						.addComponent(commentField))
				.addGroup(layout.createParallelGroup(CENTER)
						.addComponent(acceptButton)
						.addComponent(cancelButton)));
	}
	private class AddButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			Element parameterElem = document.createElement("Parameter");
			String name = nameField.getText();
			String value = valueField.getText();
			if(value.replaceAll(" ", "").length() == 0){
				JOptionPane.showMessageDialog(me, "Value field is empty",
						"Warning", JOptionPane.WARNING_MESSAGE);
				return;
			}
			parameterElem.setAttribute("Name", name);
			parameterElem.setAttribute("Value", value);
			String comment = commentField.getText();
			if(comment.length() != 0)
				parameterElem.setAttribute("Comment", comment);
			
			getElement().appendChild(parameterElem);
			
			getGraphView().setElement((Element)getElement().getParentNode());
			dispose();
		}
	}
	private class EditButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			String value = valueField.getText();
			if(value.replaceAll(" ", "").length() == 0){
				Element fbNetworkElem = (Element)getElement().getParentNode().getParentNode();
				getElement().getParentNode().removeChild(getElement());
				getGraphView().setElement(fbNetworkElem);
				dispose();
				return;
			}
			getElement().setAttribute("Value", value);
			String comment = commentField.getText();
			if(comment.length() != 0)
				getElement().setAttribute("Comment", comment);
			else
				getElement().removeAttribute("Comment");
			
			getGraphView().setElement((Element) getElement().getParentNode().getParentNode());
			dispose();
		}
	}
}
