//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.plugin;

import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.event.MenuListener;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.oooneida.wb.FBench;
import org.oooneida.wb.IconFactory;
import org.oooneida.wb.ParserLib;


/**
 * PluginManager for FBench.  General Purpose of this class is to provide easier and
 * organised access to various of FBench Plugin components.
 * 
 * @author JP
 * 
 * @version 20070530/JP
 */
public class PluginManager{
	
	private FBench fBench;
	private Hashtable<String, Object> listenerTable = new Hashtable();
	
	public PluginManager(FBench fBench){
		listenerTable.put(fBench.getClass().toString().replace("class ", ""), fBench);
		this.fBench = fBench;
	}
	
	public Vector<JMenuItem> loadMenuItems(){
		Vector<JMenuItem> menuItems = new Vector<JMenuItem>();
		Vector<Element> pluginList = loadXML();
		
		for(Element plugin : pluginList){
			Element menuHierarchy = plugin.getChild("MenuHierarchy");
			Element rootElement = menuHierarchy.getChild("Menu");
			JMenu menu = createMenuTree(rootElement);
			menuItems.add(menu);
		}
		
		return menuItems;
	}

	private JMenu createMenuTree(Element menuElement) {
		String label = menuElement.getAttributeValue("Label");
		String icon = menuElement.getAttributeValue("Icon");
		String actionCommand = menuElement.getAttributeValue("ActionCommand");
		String menuListener = menuElement.getAttributeValue("MenuListener");
		if(!listenerTable.containsKey(menuListener)){
			try {
				listenerTable.put(menuListener, 
						Class.forName(menuListener).newInstance());
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		JMenu menu = new JMenu(label);
		if(icon != null)
			menu.setIcon(IconFactory.getIcon(icon));
		menu.setActionCommand(actionCommand);
		menu.addMenuListener((MenuListener)listenerTable.get(menuListener));
		
		JMenuItem submenuItem = null;
		for(Object o : menuElement.getChildren()){
			Element submenuElement = (Element) o;
			if(submenuElement.getName().equalsIgnoreCase("Menu")){
				submenuItem = createMenuTree(submenuElement);
			}
			
			else if(submenuElement.getName().equalsIgnoreCase("MenuItem")){
				submenuItem = createMenuItem(submenuElement);
			}
			menu.add(submenuItem);
		}
		return menu;
	}

	private JMenuItem createMenuItem(Element menuElement) {
		String label = menuElement.getAttributeValue("Label");
		String icon = menuElement.getAttributeValue("Icon");
		String actionCommand = menuElement.getAttributeValue("ActionCommand");
		String actionListener = menuElement.getAttributeValue("ActionListener");
		if(!listenerTable.containsKey(actionListener)){
			try {
				listenerTable.put(actionListener, 
						Class.forName(actionListener).newInstance());
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		JMenuItem menuItem = new JMenuItem(label);
		if(icon != null) 
			menuItem.setIcon(IconFactory.getIcon(icon));
		menuItem.setActionCommand(actionCommand);
		menuItem.addActionListener((ActionListener) listenerTable.get(actionListener));
		
		return menuItem;
	}

	private Vector<Element> loadXML() {
		Vector<Element> pluginList = new Vector<Element>();
		
		File pluginRoot = new File("org\\oooneida\\wb\\plugin\\");
		File[] rootFileList = pluginRoot.listFiles();
		
		for(File file : rootFileList){
			if(file.isDirectory()){
				File[] pluginFileList = file.listFiles();
				for(File pluginFile : pluginFileList){
					if(pluginFile.getName().equals("WBPluginItem.xml")){
						try {
							Document xmlDocument = 
								ParserLib.parser(pluginFile.getAbsolutePath(), true);
							pluginList.add(xmlDocument.getRootElement());
						} catch (IOException e) {
							e.printStackTrace();
						} catch (JDOMException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
		return pluginList;
	}
}
