//Copyright (c) 2007 University of Auckland
//Contributed to the FBench extension of the OOONEIDA Workbench project under the Common Public License

package org.oooneida.wb.table;

import java.util.Vector;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.oooneida.wb.dom.ElementModel;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * 
 * 
 * @author DH
 * @version 20070130/DH
 */
public class ECActionsTableModel extends ElementModel implements TableModel {
	private NodeList actions;
	private Element stateElement;
	private Vector listeners = new Vector(4);

	/** 
	 * Constructs a table model for the ECActions of the given ECState Element.  
	 */
	public ECActionsTableModel(Element el) {
		super();
		if(el != null){
			stateElement = (Element)el.cloneNode(true);
			actions = stateElement.getChildNodes();
		}
	}

	public int getRowCount() {
		if(actions != null)
			return actions.getLength();
		return 0;
	}

	public int getColumnCount() {
		return 3;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		if(actions != null)
		switch (columnIndex) {
		case 0:
			return Integer.valueOf(rowIndex+1);
		case 1:
			return ((Element) actions.item(rowIndex)).getAttribute("Algorithm");
		case 2:
			return ((Element) actions.item(rowIndex)).getAttribute("Output");
		}
		return null;
	}

	public boolean isCellEditable(int row, int column) {
		if(column == 0)
			return false;
		return true;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		String newval = aValue.toString();
		if((getValueAt(rowIndex,columnIndex) == null) || newval.equals(getValueAt(rowIndex,columnIndex).toString())) return;
		if(actions != null){
			Element action = (Element)actions.item(rowIndex);
			if(columnIndex == 1){
				action.setAttribute("Algorithm", newval);
			}
			else if(columnIndex == 2){
				action.setAttribute("Output", newval);
			}
		}
	}

	public String getColumnName(int columnIndex) {
		switch (columnIndex) {
		case 0:
			return "";
		case 1:
			return "Algorithm";
		case 2:
			return "Output";
		}
		return "";
	}

	public Class getColumnClass(int columnIndex) {
		return String.class;
	}

	public void addTableModelListener(TableModelListener l) {
		if (!listeners.contains(l))
			listeners.addElement(l);

	}

	public void removeTableModelListener(TableModelListener l) {
		listeners.removeElement(l);
	}
	
	public Element getState(){
		return stateElement;
	}
	
	public void removeRows(int[] rows){
		for(int i = actions.getLength(); i >= 0; i--)
			for(int j = 0; j < rows.length; j++)
				if(i == rows[j])
					stateElement.removeChild(actions.item(i));
		}
}