//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.table;

import java.awt.Toolkit;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.swing.table.AbstractTableModel;

import org.oooneida.wb.dom.ElementModel;
import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.events.DocumentEvent;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * A TableModel for editing the Attributes of DOM Element.
 * 
 * @author JHC
 * @version 20051129/JHC - Eliminated use of DTDs; now sets up own attribute
 *          names.
 * @version 20050908/JHC
 */
public class ElementTableModel extends AbstractTableModel implements
		EventListener {

	/** The encapsulated Element. */
	private Element element;

	private MutationEvent mutev = null;

	/**
	 * The list of attribute names of the encapsulated Element, in the order in
	 * which they are to appear in the table.
	 */
	protected Object[] attnames = new String[0];

	private static Hashtable libraryElementAttributes = new Hashtable(64);

	private static Hashtable dataTypeAttributes = new Hashtable(32);

	/** Initializes a new TableModel for editing Elements of the given Document. */
	public ElementTableModel(Document doc) {
		super();
		mutev = (MutationEvent) ((DocumentEvent) doc)
				.createEvent("MutationEvent");
		((org.w3c.dom.events.EventTarget) doc).addEventListener("ElementSelectionEvent", this,
				false);
	}

	public int getRowCount() {
		return attnames.length;
	}

	public int getColumnCount() {
		return 2;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		if (element == null)
			return null;
		if (columnIndex == 0)
			return attnames[rowIndex];
		if (columnIndex == 1)
			return element.getAttribute(attnames[rowIndex].toString());
		return null;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (element == null) ? false : (columnIndex == 1);
	}

	public String getColumnName(int columnIndex) {
		if (columnIndex == 0)
			return "Attribute";
		if (columnIndex == 1)
			return "Value";
		return "";
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if ((element == null) || (columnIndex != 1))
			return;
		String val = aValue.toString();
		String oldval = getValueAt(rowIndex, columnIndex).toString();
		if (oldval.equals(val))
			return;
		String attname = attnames[rowIndex].toString();
		if (!attname.equals("Name") || ElementModel.isName(val)) {
			element.setAttribute(attname, val);
			fireTableCellUpdated(rowIndex, columnIndex);
			mutev.initMutationEvent("MutationEvent", true, true, element,
					oldval, val, attname, MutationEvent.MODIFICATION);
			//System.out.println("dispatching " + mutev);
			((org.w3c.dom.events.EventTarget) element).dispatchEvent(mutev);
		} else
			Toolkit.getDefaultToolkit().beep();
	}

	public void setElement(Element el) {
		if (element == el)
			return;
		element = el;
		attnames = ElementTableModel.getAttributeList(el);
		fireTableStructureChanged();
	}

	public Element getElement() {
		return element;
	}

	public void handleEvent(Event evt) {
		//System.out.println("ElementTableModel handling "+evt);
		if (evt instanceof ElementSelectionEvent)
			setElement(((ElementSelectionEvent) evt).getSelectedElement());
	}

	private static Hashtable getDataTypeAttributes() {
		if (dataTypeAttributes.isEmpty()) {
			dataTypeAttributes.put("DataType", "Name Comment");
			dataTypeAttributes.put("Identification",
					"Standard Classification ApplicationDomain Function Type Description");
			dataTypeAttributes.put("VersionInfo",
					"Organization Version Author Date Remarks");
			dataTypeAttributes.put("ASN1Tag", "Class Number");
			dataTypeAttributes.put("CompilerInfo", "header classdef");
			dataTypeAttributes.put("Compiler",
					"Language Vendor Product Version");
			dataTypeAttributes.put("DirectlyDerivedType",
					"BaseType InitialValue Comment");
			dataTypeAttributes.put("EnumeratedType",
					"InitialValue Comment");
			dataTypeAttributes.put("EnumeratedValue", "Name Comment");
			dataTypeAttributes.put("SubrangeType",
					"BaseType InitialValue Comment");
			dataTypeAttributes.put("Subrange", "LowerLimit UpperLimit");
			dataTypeAttributes.put("ArrayType",
					"BaseType InitialValues Comment");
			dataTypeAttributes.put("StructuredType", "Comment");
			dataTypeAttributes.put("VarDeclaration",
					"Name Type ArraySize InitialValue Comment");
			dataTypeAttributes.put("SubrangeVarDeclaration",
					"Name Type InitialValue Comment");
		}
		return dataTypeAttributes;
	}

	/**
	 * Returns an array of attribute name strings for the given Element.
	 */
	public static String[] getAttributeList(Element el) {
		String sysid = el.getOwnerDocument().getDoctype().getSystemId();
		Hashtable tbl = null;
		if (sysid.endsWith("LibraryElement.dtd"))
			tbl = getLibraryElementAttributes();
		else if (sysid.endsWith("DataType.dtd"))
			tbl = getDataTypeAttributes();
		else return new String[0];
		Object atts = tbl.get(el.getTagName());
		if(!(atts instanceof String)) return new String[0];
		StringTokenizer t = new StringTokenizer(atts.toString());
		String[] ans = new String[t.countTokens()];
		for (int i = 0; t.hasMoreTokens(); i++)
			ans[i] = t.nextToken();
		return ans;
	}

	private static Hashtable getLibraryElementAttributes() {
		if (libraryElementAttributes.isEmpty()) {
			libraryElementAttributes.put("FBType", "Name Comment");
			libraryElementAttributes.put("AdapterType", "Name Comment");
			libraryElementAttributes.put("ResourceType", "Name Comment");
			libraryElementAttributes.put("DeviceType", "Name Comment");
			libraryElementAttributes.put("System", "Name Comment");
			libraryElementAttributes.put("SubAppType", "Name Comment");
			libraryElementAttributes
					.put("Identification",
							"Standard Classification ApplicationDomain Function Type Description");
			libraryElementAttributes.put("VersionInfo",
					"Organization Version Author Date Remarks");
			libraryElementAttributes.put("CompilerInfo", "header classdef");
			libraryElementAttributes.put("Compiler",
					"Language Vendor Product Version");
			libraryElementAttributes.put("FB", "Name Type Comment x y");
			libraryElementAttributes.put("Connection",
					"Source Destination Comment dx1 dx2 dy");
			libraryElementAttributes.put("Event", "Name Type Comment");
			libraryElementAttributes.put("With", "Var");
			libraryElementAttributes.put("VarDeclaration",
					"Name Type ArraySize InitialValue Comment");
			libraryElementAttributes.put("AdapterDeclaration",
					"Name Type Comment x y");
			libraryElementAttributes.put("ECState", "Name Comment x y");
			libraryElementAttributes.put("ECTransition",
					"Source Destination Condition Comment x y");
			libraryElementAttributes.put("ECAction", "Algorithm Output");
			libraryElementAttributes.put("Algorithm", "Name Comment");
			libraryElementAttributes.put("ST", "Text");
			libraryElementAttributes.put("Rung", "Output Expression Comment");
			libraryElementAttributes.put("Other", "Language Text");
			libraryElementAttributes.put("Service",
					"RightInterface LeftInterface Comment");
			libraryElementAttributes.put("ServiceSequence", "Name Comment");
			libraryElementAttributes.put("InputPrimitive",
					"Interface Event Parameters");
			libraryElementAttributes.put("OutputPrimitive",
					"Interface Event Parameters");
			libraryElementAttributes.put("FBTypeName", "Name");
			libraryElementAttributes.put("ResourceTypeName", "Name");
			libraryElementAttributes.put("Resource", "Name Type Comment x y");
			libraryElementAttributes.put("Application", "Name Comment");
			libraryElementAttributes.put("Mapping", "From To");
			libraryElementAttributes.put("Device", "Name Type Comment x y");
			libraryElementAttributes.put("SubAppEvent", "Name Type Comment");
			libraryElementAttributes.put("SubApp", "Name Type Comment x y");
			libraryElementAttributes
					.put("Segment", "Name Type Comment x y dx1");
			libraryElementAttributes.put("Parameter", "Name Value Comment");
			libraryElementAttributes.put("Link",
					"SegmentName CommResource Comment");
		}
		return libraryElementAttributes;
	}
}
