package org.oooneida.wb.translate;

import java.io.*;
import java.lang.*;

%%

%class STLex
%line
%char
%state COMMENT
%byaccj
%full

%{
  //////////////////////////////
  private boolean DEBUG = false;
  //////////////////////////////

  private int comment_count = 0;
  StringBuffer string = new StringBuffer();

  private STtoJava yySTtoJava;

  public STLex(java.io.Reader r, STtoJava yySTtoJava) {
    this(r);
    this.yySTtoJava = yySTtoJava;
  }
  
  private long parseLong(int start, int end, int radix) {
    long result = 0;
    long digit;

    for (int i = start; i < end; i++) {
      digit  = Character.digit(yycharat(i),radix);
      result*= radix;
      result+= digit;
    }

    if(DEBUG) System.out.print("Comma"); return result;
  }

%} 

ALPHA			= [A-Za-z]
DIGIT			= [0-9]

/* main character classes */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]

WhiteSpace = {LineTerminator} | [ \t\f]

/* comments */
Comment = {TraditionalComment} | {EndOfLineComment} | 
          {DocumentationComment}

TraditionalComment = "(*" [^*] ~"*)" | "(*" "*"+ ")"
EndOfLineComment = "//" {InputCharacter}*
DocumentationComment = "(*" "*"+ [^/*] ~"*)"

/* identifiers */
Identifier = {ALPHA}({ALPHA}|{DIGIT}|_|\.)*

/* integer literals */
DecIntegerLiteral = 0 | [1-9][0-9]*
DecLongLiteral    = {DecIntegerLiteral} [lL]

HexIntegerLiteral = 0 [xX] 0* {HexDigit} {1,8}
HexLongLiteral    = 0 [xX] 0* {HexDigit} {1,16} [lL]
HexDigit          = [0-9a-fA-F]

OctIntegerLiteral = 0+ [1-3]? {OctDigit} {1,15}
OctLongLiteral    = 0+ 1? {OctDigit} {1,21} [lL]
OctDigit          = [0-7]
    
/* floating point literals */        
FloatLiteral  = ({FLit1}|{FLit2}|{FLit3}) {Exponent}? [fF]
DoubleLiteral = ({FLit1}|{FLit2}|{FLit3}) {Exponent}?

FLit1    = [0-9]+ [1\.][^\.][0-9]* 
FLit2    = [^\.][1\.] [0-9]+ 
FLit3    = [0-9]+ 
Exponent = [eE] [+-]? [0-9]+

/* string and character literals */
StringCharacter = [^\r\n\"\\]
SingleCharacter = [^\r\n\'\\]

%state STRING, CHARLITERAL

%% 

<YYINITIAL> {
  ","  { if(DEBUG) System.out.print("Comma"); return STtoJava.Comma; }
  ":"  { if(DEBUG) System.out.print("Colon"); return STtoJava.Colon; }
  ";"  { if(DEBUG) System.out.print("Semicolon"); return STtoJava.SemiColon; }
  "("  { if(DEBUG) System.out.print("OParen"); return STtoJava.OParen; }
  ")"  { if(DEBUG) System.out.print("CParen"); return STtoJava.CParen; }
  "["  { if(DEBUG) System.out.print("OBracket"); return STtoJava.OBracket; }
  "]"  { if(DEBUG) System.out.print("CBracket"); return STtoJava.CBracket; }
  "{"  { if(DEBUG) System.out.print("OBrace"); return STtoJava.OBrace; }
  "}"  { if(DEBUG) System.out.print("CBrace"); return STtoJava.CBrace; }
  ".." { if(DEBUG) System.out.print("Range"); return STtoJava.Range; }
  "."  { if(DEBUG) System.out.print("Point"); return STtoJava.Point; }
  "+"  { if(DEBUG) System.out.print("Plus"); return STtoJava.Plus; }
  "-"  { if(DEBUG) System.out.print("Minus"); return STtoJava.Minus; }
  "**"  { if(DEBUG) System.out.print("Power"); return STtoJava.Power; }
  "*"  { if(DEBUG) System.out.print("Asterisk"); return STtoJava.Asterisk; }
  "/"  { if(DEBUG) System.out.print("FSlash"); return STtoJava.FSlash; }
  "="  { if(DEBUG) System.out.print("Equal"); return STtoJava.Equal; }
  "<>" { if(DEBUG) System.out.print("NotEqual"); return STtoJava.NotEqual; }
  "<"  { if(DEBUG) System.out.print("LessThan"); return STtoJava.LessThan; }
  "<=" { if(DEBUG) System.out.print("LessThanEqualTo"); return STtoJava.LessThanEqualTo; }
  ">"  { if(DEBUG) System.out.print("GreaterThan"); return STtoJava.GreaterThan; }
  ">=" { if(DEBUG) System.out.print("GreaterThanEqualTo"); return STtoJava.GreaterThanEqualTo; }
  "&"  { if(DEBUG) System.out.print("And"); return STtoJava.And; }
  "|"  { if(DEBUG) System.out.print("Or"); return STtoJava.Or; }
  ":=" { if(DEBUG) System.out.print("Assign"); return STtoJava.Assign; }
  "%" { if(DEBUG) System.out.print("DirectRep"); return STtoJava.DirectRep; }
  "#" { if(DEBUG) System.out.print("NumSeparator"); return STtoJava.NumSeparator; }
  "!" { if(DEBUG) System.out.print("Or"); System.out.println("Warning: '!' is not allowed in this character set."); return STtoJava.Or; }
  
  "TRUE" { if(DEBUG) System.out.print("True"); yySTtoJava.yylval = new STtoJavaVal("true"); return STtoJava.BooleanLiteral; }
  "FALSE" { if(DEBUG) System.out.print("False"); yySTtoJava.yylval = new STtoJavaVal("false"); return STtoJava.BooleanLiteral; }
  
  "NOT" { if(DEBUG) System.out.print("Not"); return STtoJava.Not; }
  "AND" { if(DEBUG) System.out.print("And"); return STtoJava.And; }
  "XOR" { if(DEBUG) System.out.print("Xor"); return STtoJava.Xor; }
  "OR" { if(DEBUG) System.out.print("Or"); return STtoJava.Or; }
  "MOD" { if(DEBUG) System.out.print("Mod"); return STtoJava.Mod; }

  "TYPE" { if(DEBUG) System.out.print("Type"); return STtoJava.Type; }
  "OF" { if(DEBUG) System.out.print("Of"); return STtoJava.Of; }
  "STRUCT" { if(DEBUG) System.out.print("Struct"); return STtoJava.Struct; }
  "END_STRUCT" { if(DEBUG) System.out.print("EndStruct"); return STtoJava.EndStruct; }
  "END_TYPE" { if(DEBUG) System.out.print("EndType"); return STtoJava.EndType; }
  
  "FUNCTION" { if(DEBUG) System.out.print("Function"); return STtoJava.Function; }
  "VAR_INPUT" { if(DEBUG) System.out.print("VarInput"); return STtoJava.VarInput; }
  "VAR_OUTPUT" { if(DEBUG) System.out.print("VarOutput"); return STtoJava.VarOutput; }
  "VAR_IN_OUT" { if(DEBUG) System.out.print("VarInputOutput"); return STtoJava.VarInputOutput; }
  "VAR_EXTERNAL" { if(DEBUG) System.out.print("VarExternal"); return STtoJava.VarExternal; }
  "VAR_ACCESS" { if(DEBUG) System.out.print("VarAccess"); return STtoJava.VarAccess; }
  "VAR_GLOBAL" { if(DEBUG) System.out.print("VarGlobal"); return STtoJava.VarGlobal; }
  "END_VAR" { if(DEBUG) System.out.print("EndVar"); return STtoJava.EndVar; }
  "RETURN" { if(DEBUG) System.out.print("Return"); return STtoJava.Return; }
  "END_FUNCTION" { if(DEBUG) System.out.print("EndFunction"); return STtoJava.EndFunction; }
  
  "AT" { if(DEBUG) System.out.print("At"); return STtoJava.At; }
  "EN" { if(DEBUG) System.out.print("En"); return STtoJava.En; }
  "ENO" { if(DEBUG) System.out.print("Eno"); return STtoJava.Eno; }
  "F_EDGE" { if(DEBUG) System.out.print("FEdge"); return STtoJava.FEdge; }
  "R_EDGE" { if(DEBUG) System.out.print("REdge"); return STtoJava.REdge; }
  "CONSTANT" { if(DEBUG) System.out.print("Constant"); return STtoJava.Constant; }
  "PROGRAM" { if(DEBUG) System.out.print("Program"); return STtoJava.Program; }
  "END_PROGRAM" { if(DEBUG) System.out.print("EndProgram"); return STtoJava.EndProgram; }
  "WITH" { if(DEBUG) System.out.print("With"); return STtoJava.With; }
  "READ_ONLY" { if(DEBUG) System.out.print("ReadOnly"); return STtoJava.ReadOnly; }
  "READ_WRITE" { if(DEBUG) System.out.print("ReadWrite"); return STtoJava.ReadWrite; }
  "RESOURCE" { if(DEBUG) System.out.print("Resource"); return STtoJava.Resource; }
  "END_RESOURCE" { if(DEBUG) System.out.print("EndResource"); return STtoJava.EndResource; }
  "RETAIN" { if(DEBUG) System.out.print("Retain"); return STtoJava.Retain; }
  "STEP" { if(DEBUG) System.out.print("Step"); return STtoJava.Step; }
  "END_STEP" { if(DEBUG) System.out.print("EndStep"); return STtoJava.EndStep; }
  "TASK" { if(DEBUG) System.out.print("Task"); return STtoJava.Task; }
  "TRANSITION" { if(DEBUG) System.out.print("Transition"); return STtoJava.Transition; }
  "END_TRANSITION" { if(DEBUG) System.out.print("EndTransition"); return STtoJava.EndTransition; }
  "FROM" { if(DEBUG) System.out.print("From"); return STtoJava.From; }
  
  "INITIAL_STEP" { if(DEBUG) System.out.print("InitialStep"); return STtoJava.InitialStep; }
  "END_STEP" { if(DEBUG) System.out.print("EndStep"); return STtoJava.EndStep; }
  
  "ACTION" { if(DEBUG) System.out.print("Action"); return STtoJava.Action; }
  "END_ACTION" { if(DEBUG) System.out.print("EndAction"); return STtoJava.EndAction; }
  
  "ARRAY" { if(DEBUG) System.out.print("Array"); return STtoJava.Array; }
  
  "IF" { if(DEBUG) System.out.print("If"); return STtoJava.If; }
  "THEN" { if(DEBUG) System.out.print("Then"); return STtoJava.Then; }
  "ELSIF" { if(DEBUG) System.out.print("ElseIf"); return STtoJava.ElseIf; }
  "ELSE" { if(DEBUG) System.out.print("Else"); return STtoJava.Else; }
  "END_IF" { if(DEBUG) System.out.print("EndIf"); return STtoJava.EndIf; }
  
  "CASE" { if(DEBUG) System.out.print("Case"); return STtoJava.Case; }
  "END_CASE" { if(DEBUG) System.out.print("EndCase"); return STtoJava.EndCase; }
  
  "FOR" { if(DEBUG) System.out.print("For"); return STtoJava.For; }
  "TO" { if(DEBUG) System.out.print("To"); return STtoJava.To; }
  "BY" { if(DEBUG) System.out.print("By"); return STtoJava.By; }
  "DO" { if(DEBUG) System.out.print("To"); return STtoJava.Do; }
  "END_FOR" { if(DEBUG) System.out.print("EndFor"); return STtoJava.EndFor; }
  "WHILE" { if(DEBUG) System.out.print("While"); return STtoJava.While; }
  "END_WHILE" { if(DEBUG) System.out.print("EndWhile"); return STtoJava.EndWhile; }
  "REPEAT" { if(DEBUG) System.out.print("Repeat"); return STtoJava.Repeat; }
  "UNTIL" { if(DEBUG) System.out.print("Until"); return STtoJava.Until; }
  "EXIT" { if(DEBUG) System.out.print("Exit"); return STtoJava.Exit; }

  /* string literal */
  \"                             { yybegin(STRING); string.setLength(0); }

  /* character literal */
  \'                             { yybegin(CHARLITERAL); }

  /* numeric literals */

  {DecIntegerLiteral}            { yySTtoJava.yylval = new STtoJavaVal(yytext()); if(DEBUG) System.out.print("IntegerLiteral"); return STtoJava.IntegerLiteral; }
  {DecLongLiteral}               { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(0,yylength()-1)); if(DEBUG) System.out.print("IntegerLiteral"); return STtoJava.LongLiteral; }
  
  {HexIntegerLiteral}            { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(2, yylength() - 2)); if(DEBUG) System.out.print("IntegerLiteral"); return STtoJava.IntegerLiteral; }
  {HexLongLiteral}               { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(2, yylength() - 3)); if(DEBUG) System.out.print("LongLiteral"); return STtoJava.LongLiteral; }
 
  {OctIntegerLiteral}            { yySTtoJava.yylval = new STtoJavaVal(yytext()); if(DEBUG) System.out.print("IntegerLiteral"); return STtoJava.IntegerLiteral; }
  {OctLongLiteral}               { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(0,yylength()-1)); if(DEBUG) System.out.print("LongLiteral"); return STtoJava.LongLiteral; }
  
  {FloatLiteral}                 { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(0,yylength()-1)); if(DEBUG) System.out.print("FloatLiteral1"); return STtoJava.FloatLiteral; }
  {DoubleLiteral}                { yySTtoJava.yylval = new STtoJavaVal(yytext()); if(DEBUG) System.out.print("FloatLiteral2"); return STtoJava.FloatLiteral; }
  {DoubleLiteral}[dD]            { yySTtoJava.yylval = new STtoJavaVal(yytext().substring(0,yylength()-1)); if(DEBUG) System.out.print("FloatLiteral"); return STtoJava.FloatLiteral; }
  
  /* comments */
  {Comment}                      { yySTtoJava.yylval = new STtoJavaVal(yytext()); if(DEBUG) System.out.print("CommentText"); return STtoJava.CommentText; }

  /* whitespace */
  {LineTerminator}				 { if(DEBUG) System.out.print(yytext()); return STtoJava.NewLine; }
  {WhiteSpace}                   { if(DEBUG) System.out.print(yytext()); }

  /* identifiers */ 
  {Identifier}                   { yySTtoJava.yylval = new STtoJavaVal(yytext()); if(DEBUG) System.out.print("Identifier"); return STtoJava.Identifier; } 
}

<STRING> {
  \"                             { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal(string.toString()); if(DEBUG) System.out.print("StringLiteral"); return STtoJava.StringLiteral; }
  
  {StringCharacter}+             { string.append( yytext() ); }
  
  /* escape sequences */
  "\\b"                          { string.append( '\b' ); }
  "\\t"                          { string.append( '\t' ); }
  "\\n"                          { string.append( '\n' ); }
  "\\f"                          { string.append( '\f' ); }
  "\\r"                          { string.append( '\r' ); }
  "\\\""                         { string.append( '\"' ); }
  "\\'"                          { string.append( '\'' ); }
  "\\\\"                         { string.append( '\\' ); }
  \\[0-3]?{OctDigit}?{OctDigit}  { char val = (char) Integer.parseInt(yytext().substring(1),8);
                        				   string.append( val ); }
  
  /* error cases */
  \\.                            { throw new RuntimeException("Illegal escape sequence \""+yytext()+"\""); }
  {LineTerminator}               { throw new RuntimeException("Unterminated string at end of line"); }
}

<CHARLITERAL> {
  {SingleCharacter}\'            { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("" + yytext().charAt(0)); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  
  /* escape sequences */
  '\\b'\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\b"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\t'\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\t"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\n'\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\n"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\f'\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\f"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\r'\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\r"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\\''\'                       { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\'"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\''\'                        { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\'"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  '\\\\'\'                       { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal("\\"); if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  \\[0-3]?{OctDigit}?{OctDigit}\' { yybegin(YYINITIAL); yySTtoJava.yylval = new STtoJavaVal(Integer.parseInt(yytext().substring(1,yylength()-1),8));
									if(DEBUG) System.out.print("CharLiteral"); return STtoJava.CharLiteral; }
  
  /* error cases */
  \\.                            { throw new RuntimeException("Illegal escape sequence \""+yytext()+"\""); }
  {LineTerminator}               { throw new RuntimeException("Unterminated character literal at end of line"); }
}

. {
	System.err.print("Illegal character: <" + yytext() + ">");
	return STtoJava.Error;
}