%{
  import java.io.*;
  import java.util.*;
  import java.util.regex.*;
  import org.oooneida.wb.translate.Translator;
%}

%start ProgramSource
      
%token NewLine, WhiteSpace, Not, And, Or, Mod, Xor, Power, Type, Of, Struct,
	Array, EndStruct, Function, VarInput, VarOutput, VarInputOutput, EndVar, EndType, Return, EndFunction,
	If, Then, Else, ElseIf, EndIf, Case, EndCase, For, To, By, EndFor, While, Do, EndWhile, Repeat, Until, 
	Exit, Comma, Colon, SemiColon, OParen, CParen, OBracket, CBracket, OBrace, CBrace, Point, Plus, Minus, 
	Asterisk, FSlash, Equal, NotEqual, LessThan, LessThanEqualTo, GreaterThan, GreaterThanEqualTo, Assign, Range,
	Error, NumSeparator, DirectRep, Action, EndAction, At, Constant, En, Eno, FEdge, InitialStep, EndStep,
	Program, With, EndProgram, REdge, ReadOnly, ReadWrite, Resource, EndResource, Retain, Step, EndStep,
	Task, Transition, EndTransition, From, VarExternal, VarAccess, VarGlobal
	
%token <sval> BooleanLiteral
%token <sval> IntegerLiteral, LongLiteral
%token <sval> FloatLiteral

%token <sval> Identifier, UnclosedStringLiteral, CommentText, StringLiteral, CharLiteral

%type <sval> ProgramSource, Expression, number, Statement, StatementList, Declaration, AssignmentStatement, SubprogramControlStatement,
	FlowStatement, IterationStatement, FBInvocation, FBInputs, FBInputAssignment, FlowStatement, IfStatement,
	CaseStatement, CaseElements, CaseElseStatement, CaseElement, CaseList, CaseListElement,
	IterationStatement, ForStatement, ForList, WhileStatement, RepeatStatement, Subrange, TypeDefinition, TypeBlock,
	StructType, StructElements, StructElement, EnumType, EnumList, SubrangeType, ArrayType, ArrayElementList, AssignExpression,
	AssignArray, AssignFunction, AssignArrayElements, AssignArrayElement, FunctionHeader, FunctionBlock, FunctionDefinition, 
	VarInputs, VarInOuts, VarOutputs, VarList, Vars

%left Or, Xor
%left And
%left Equal NotEqual GreaterThan LessThan GreaterThanEqualTo LessThanEqualTo
%left Minus Plus
%left Asterisk FSlash Mod
%left UNARY        /* precedence for unary minus, not */
%right Power         /* exponentiation        */

%%

Declaration : Identifier Identifier			{ $$ = $1 + " " + $2 + " = new " + $1 + "();"; }
			 | Identifier Identifier Assign Expression { $$ = $1 + " " + $2 + " = new " + $1 + "(" + $4 + ");"; }
			 | Identifier AssignFunction	{ $$ = $1 + " " + $2 + ";"; } 
			 ;

AssignmentStatement : AssignExpression		{ $$ = $1 + ";"; }
						| AssignArray		{ $$ = $1; }
						| AssignFunction	{ $$ = $1 + ";"; }
						;
						
AssignExpression : Identifier Assign Expression		{ $$ = $1 + ".value = " + $3; }
					;
					
AssignArray : Identifier Assign AssignArrayElements { 
														String result = "";
														String[] elems = $3.split(",");
														int totalElements = 0;
														for(int i = 0; i < elems.length; i++){
															String strElemCount = elems[i].substring(0, elems[i].indexOf('('));
															String strElemValue = elems[i].substring(elems[i].indexOf('('));
															strElemValue = strElemValue.substring(1, strElemValue.length() - 1);
															//System.out.println("\nArray strElemCount: " + strElemCount);
															//System.out.println("\nArray strElemValue: " + strElemValue);
															
															int count = Integer.parseInt(strElemCount.trim());
															int lt = totalElements + count;
															
															if(i > 0) result += "\n";
															
															result += "for(int i = " + totalElements + "; i < " + lt + 
															 "; i++){ " + $1 + "[i] = " + strElemValue + "; }";
															
															totalElements += count;
														}
														
														//System.out.println("\nArray result: " + result);
														
														//$$ = $1 + " = " + $3; 
														$$ = result; 
													};

AssignArrayElements : AssignArrayElement			{ $$ = $1; }
					| AssignArrayElements Comma AssignArrayElement { $$ = $1 + ", " + $3; }
					;

AssignArrayElement : IntegerLiteral OParen IntegerLiteral CParen { $$ = $1 + "(" + $3 + ")"; }
					;

AssignFunction : Identifier Assign FBInvocation	{ $$ = $1 + " = " + $3; }
					;
						
TypeDefinition : Type Identifier Colon NewLine TypeBlock EndType 
											{ 
												$$ = $5.replace("###", $2); 
												
												if(addArrayDef){
													arrayTypes.put($2, arrayDefType);
													arrayInstantiates.put($2, arrayDefInstantiate);
													addArrayDef = false;
												}
											}
				;

TypeBlock : StructType						{ $$ = $1; }
			| EnumType						{ $$ = $1; }
			| SubrangeType					{ $$ = $1; }
			| ArrayType						{ $$ = $1; }
			;
			
StructType : Struct NewLine StructElements EndStruct SemiColon NewLine { $$ = "class ### {\n" + $3 + "\n}"; }
			;

StructElements : StructElement				{ $$ = $1; }
				| StructElements StructElement { $$ = $1 + $2; }
				;

StructElement : Identifier Colon Identifier SemiColon NewLine { $$ = "public " + $3 + " " + $1 + " = new " + $3 + "();\n"; }
				;

EnumType : OParen EnumList CParen SemiColon NewLine { $$ = "enum ### { " + $2 + " }"; }
			;

EnumList : Identifier						{ $$ = $1; }
			| EnumList Comma Identifier		{ $$ = $1 + ", " + $3; }
			;


SubrangeType : Identifier OParen number Range number CParen SemiColon NewLine 
				{ 
					System.err.println("(" + lineNo + ") Subrange types not implemented yet."); 
					addError(lineNo, "Subrange types not implemented.", true); 
					$$ = "/* Subrange types not implemented yet.\nTYPE ### :\n" + $1 + "(" + $3 + ".." + $5 + ");\nEND_TYPE\n*/"; 
				}
				;

ArrayType : Array OBracket ArrayElementList CBracket Of Identifier SemiColon NewLine 
				{  
					String type = $6;
					String instantiate = type;
					String[] dims = $3.split(",");
					for(int i = 0; i < dims.length; i++){
						String size = dims[i].substring(dims[i].lastIndexOf('.'));
						size = size.substring(1, size.length());
						type += "[]";
						instantiate += "[" + size + "]";
					}
					
					arrayDefInstantiate = instantiate;
					arrayDefType = type;
					addArrayDef = true;
					
					System.out.println("(" + lineNo + ") Array types not implemented, substituting java array " + type);
					addError(lineNo, "### = " + type, false);
					$$ = "";
					//System.err.println("(" + lineNo + ") Array types not implemented yet.");
					//addError(lineNo, "Array types not implemented.", true); 
					//$$ = "/* Array types not implemented yet.\nTYPE ### :\nARRAY[" + $3 + "] OF " + $6 + ";\nEND_TYPE\n*/"; 
				} 
				;

ArrayElementList : Subrange					{ $$ = $1; }
				| ArrayElementList Comma Subrange { $$ = $1 + ", " + $3; }
				;

SubprogramControlStatement : FBInvocation   { $$ = $1 + ";"; }
							| Return		{ $$ = "return;"; }
							;

FBInvocation : Identifier OParen FBInputs CParen	{ $$ = $1 + "(" + $3 + ")"; }
				| And OParen FBInputs CParen		{ $$ = opFuntion("&&", $3); }
				| Or OParen FBInputs CParen			{ $$ = opFuntion("||", $3); }
				| Xor OParen FBInputs CParen		{ $$ = opFuntion("^", $3); }
				;
				
FBInputs :  FBInputAssignment				{ $$ = $1; }
			| FBInputs Comma FBInputAssignment	{ $$ = $1 + ", " + $3; }
			;
			
FBInputAssignment : Expression				{ $$ = $1; }
					| Identifier Assign Expression   
											{ 
												System.err.println("(" + lineNo + ") Named parameters are not supported.");
												addError(lineNo, "Named parameters are not supported.", true); 
												$$ = "/* Named parameters are not supported: " + $1 + " := " + $3 + "*/"; 
											}
					;
					
FunctionDefinition : Function Identifier Colon Identifier NewLine FunctionBlock EndFunction
											{
												String[] parts = $6.split("###");
												String header = parts[0].trim();
												String block = parts[1].trim();
												
												$$ = $4 + " " + $2 + "(" + header + "){\n" + $4 + " " + $2 + " = new " + $4 + "();\n" + 
													block + "\nreturn " + $2 + ";\n}"; 
											} ;
				
FunctionBlock : FunctionHeader StatementList 
											{ 
											  addError(lineNo, "Functions cannot be declared inside algorithms.", false);
											  $$ = $1 + "###" + $2; 
											}
				;
				
FunctionHeader : VarInputs					{ $$ = $1; }
				| VarOutputs				{ 
												System.out.println("(" + lineNo + ") Output vars not supported."); 
												addError(lineNo, "Output vars not supported.", true);
												$$ = "";
											}
				| VarInOuts					{ 
												System.out.println("(" + lineNo + ") InputOutput vars not supported."); 
												addError(lineNo, "InputOutput vars not supported.", true);
												$$ = ""; 
											}
				| FunctionHeader NewLine VarInOuts	{ $$ = $1 + "\n" + $3; }
				| FunctionHeader NewLine VarInputs	{ $$ = $1 + "\n" + $3; }
				| FunctionHeader NewLine VarOutputs { $$ = $1 + "\n" + $3; }
				;
				
VarInputs : VarInput NewLine VarList NewLine EndVar { $$ = $3; }
				;
									
VarOutputs : VarOutput NewLine VarList NewLine EndVar { $$ = ""; }
				;
VarInOuts : VarInputOutput NewLine VarList NewLine EndVar { $$ = ""; }
				;

VarList : Vars Colon Identifier	SemiColon	{ $$ = varList($3, $1); }
			| VarList NewLine VarList		{ $$ = $1 + "\n" + $3; }
			;

Vars : Identifier							{ $$ = $1; }
			| Vars Comma Vars				{ $$ = $1 + ", " + $3; }
			;

FlowStatement : IfStatement					{ $$ = $1; }
			| CaseStatement					{ $$ = $1; }
			;
					 
IfStatement : 
			If Expression Then StatementList EndIf
											{ $$ = "if (" + $2 + "){" +$4 + "}"; }
			| If Expression Then StatementList Else StatementList EndIf 
											{ $$ = "if (" + $2 + "){" + $4 + "} else {" + $6 + "}"; }
			| If Expression Then StatementList ElseIf Expression Then StatementList EndIf 
											{ $$ = "if (" + $2 + "){" + $4 + "} else if(" + $6 + ") {" + $8 + "}"; }
			| If Expression Then StatementList ElseIf Expression Then StatementList Else StatementList EndIf
											{ $$ = "if (" + $2 + "){" + $4 + "} else if(" + $6 + ") {" + $8 + "} else {" + $10 + "}"; }
				;
					
CaseStatement :		Case Expression Of NewLine CaseElements CaseElseStatement EndCase 
											{ $$ = "switch (" + $2 + "){" + $5 + $6 + "}"; }
					;
					
CaseElements :		CaseElement				{ $$ = $1; }
					| CaseElements CaseElement { $$ = $1 + $2; }
					;
					
CaseElseStatement	: NewLine				{ $$ = ""; }
					| Else StatementList	{ $$ = "default: " + $2; }
					;
                  
CaseElement :		CaseList Colon StatementList { String block = $3; if(block.length() > 0) block += "break;\n"; $$ = $1 + block; }
					;
					
CaseList :			CaseListElement			{ $$ = $1; }
					| CaseList Comma CaseListElement { $$ = "\n" + $3; }
					;
					
CaseListElement :	Subrange				{ 
												// Substitute list of cases
												String[] parts = $1.split("\\.\\.");
												int start = Integer.parseInt(parts[0]);
												int end = Integer.parseInt(parts[1]);
												String result = "";
												for(int i = start; i < end - start; i++)
													result += "case " + i + ":\n";
												$$ = result;
											}
					| IntegerLiteral		{ $$ = "case " + $1 + ": "; }
					;

IterationStatement : ForStatement			{ $$ = $1; }
					 | WhileStatement		{ $$ = $1; }
					 | RepeatStatement		{ $$ = $1; }
					 | Exit					{ $$ = "break"; }
					 ;

ForStatement : For Identifier Assign ForList Do StatementList EndFor 
				{ String temp = $4.replace("###", $2 + ".value"); $$ = "for(" + $2 + ".value = " + temp + "){" + $6 + "}"; }
				;
				
ForList : Expression To Expression			{ $$ = $1 + "; ### <= " + $3 + "; ###++"; }
		| Expression To Expression By Expression	{ $$ = $1 + "; ### != " + $3 + "; ### = ###" + ($5.charAt(0) == '-' ? $5 : "+" + $5); }
		;
		
WhileStatement : While Expression Do StatementList EndWhile 
											{ $$ = "while(" + $2 + "){" + $4 + "}"; }
				;

RepeatStatement : Repeat StatementList Until Expression 
											{ 
												System.err.println("(" + lineNo + ") Java does not support do-util.");
												addError(lineNo, "Java does not support do-until.", false);
												$$ = "/* Warning: Java does not support do-until, please ensure the following block is correct: */\n" + 
												"do {" + $2 + "} while(!(" + $4 + "))" + 
												"\n/* Original expression: " + $4 + " */"; 
											}
					;
				
Statement : NewLine							{ lineNo++; $$ = "\n"; }
			| Declaration SemiColon			{ $$ = $1; }
			| TypeDefinition				{ $$ = $1; }
			| FunctionDefinition			{ $$ = $1; }
			| AssignmentStatement SemiColon	{ $$ = $1; }
			| SubprogramControlStatement SemiColon	{ $$ = $1; }
			| FlowStatement SemiColon		{ $$ = $1; }
			| IterationStatement SemiColon	{ $$ = $1; }
			;

ProgramSource : StatementList 
			{
				String code = postProcess($1);
				source.append(code); 
			}
			;

StatementList : CommentText					{ $$ = $1; }
				| Statement					{ $$ = $1; }
				| Statement CommentText		{ $$ = $1; }
				| StatementList Statement	{ $$ = $1 + $2; }
				| StatementList Statement CommentText { $$ = $1 + $2; }
				| Expression				{ $$ = $1; } // For guard conditions
				;

Subrange : number Range number				{ $$ = $1 + ".." + $3; }
			;
      
number : IntegerLiteral						{ $$ = $1; }
		 | LongLiteral						{ $$ = $1; }
		 | FloatLiteral						{ if($1.trim().charAt(0) == '.') $$ = "0" + $1.trim(); else $$ = $1; }
		 ;
			
Expression : BooleanLiteral					{ $$ = $1; }
			| Identifier					{ $$ = $1 + ".value"; }
			| number						{ $$ = $1; }
			| Expression And Expression		{ $$ = $1 + " && " + $3; }
			| Expression Plus Expression	{ $$ = $1 + " + " + $3; }
			| Expression Minus Expression	{ $$ = $1 + " - " + $3; }
			| Expression Asterisk Expression { $$ = $1 + " * " + $3; }
			| Expression FSlash Expression	{ $$ = $1 + " / " + $3; }
			| Expression Mod Expression		{ $$ = $1 + " % " + $3; }
			| Expression Power Expression	{ $$ = "Math.pow(" + $1 + ", " + $3 + ")"; }
			| Expression Or Expression		{ $$ = $1 + " || " + $3; }
			| Expression Xor Expression		{ $$ = $1 + " ^ " + $3; }
			| Expression Equal Expression	{ $$ = $1 + " == " + $3; }
			| Expression NotEqual Expression { $$ = $1 + " != " + $3; }
			| Expression LessThan Expression { $$ = $1 + " < " + $3; }
			| Expression LessThanEqualTo Expression	{ $$ = $1 + " <= " + $3; }
			| Expression GreaterThan Expression { $$ = $1 + " > " + $3; }
			| Expression GreaterThanEqualTo Expression	{ $$ = $1 + " >= " + $3; }
			| Minus Expression %prec UNARY	{ $$ = "-" + $2; }
			| Not Expression %prec UNARY	{ $$ = "!" + $2; }
			| OParen Expression CParen		{ $$ = "(" + $2 + ")"; }
			;


%%
  
  private int lineNo = 0;
  private STLex lexer;
  private ArrayList<ParseError> errors = new ArrayList<ParseError>();
  private Hashtable<String, String> arrayTypes = new Hashtable<String, String>();
  private Hashtable<String, String> arrayInstantiates = new Hashtable<String, String>();
  private StringBuilder source = new StringBuilder();
  private boolean addArrayDef;
  private String arrayDefType;
  private String arrayDefInstantiate;

  private int yylex () {
    int yyl_return = -1;
    try {
      yylval = new STtoJavaVal(0);
      yyl_return = lexer.yylex();
    }
    catch (IOException e) {
      System.err.println("IO error :"+e);
    }
    return yyl_return;
  }
  
  private String postProcess(String code){  
    // Replace all array instantations with valid Java
	for(String s : arrayInstantiates.keySet()){
		int index = 0;
		while((index = code.indexOf("new " + s + "()")) > 0){
			String first = code.substring(0, index);
			String second = code.substring(index + ("new " + s + "()").length());
			code = first + "new " + arrayInstantiates.get(s) + second;
		}
	}

	// Replace all array types
	for(String s : arrayTypes.keySet()){
		int index = 0;
		while((index = code.indexOf(s)) > 0){
			String first = code.substring(0, index);
			String second = code.substring(index + s.length());
			code = first + arrayTypes.get(s) + second;
		}
		code.replaceAll(s, arrayTypes.get(s));
		System.out.println(s + " > " + arrayTypes.get(s));
	}
	
	return code;
  }
  
  /**
   * Adds an error to the result.
   * 
   */
  public void addError(int line, String message, boolean critical){
    try{
	  errors.add(new ParseError(line, message, critical));
	} catch (Exception e) { /**/ }
  }

  /**
   * Returns the Java source.
   * 
   */
  public String getSource(){
    return source.toString();
  }

  /**
   * Returns a list of errors encountered.
   * 
   */
  public ArrayList<ParseError> getErrors(){
    return errors;
  }
  
  /**
   * Gets the parser result.
   * 
   */
  public ParseResult getResult(){
    return new ParseResult(getSource(), getErrors());
  }

  /**
   * Internal, prints an error message.
   * 
   */
  public void yyerror (String error) {
    //System.err.println ("Error: " + error);
    addError(getLine(), error, true);
  }
  
  /**
   * Returns the current line being parsed.
   * 
   */
  public int getLine(){
	return lineNo;
  }

  private String opFuntion(String operator, String arguments){
    // In ST a boolean operator such as AND or OR can be used as a function.
	// This method inserts an operator between a list of arguments to produce
	// the same effect in Java.
    //System.out.println("  OPString: " + operator + " + " + arguments);
    String[] args = arguments.split(",");
    String result = "";
    for(int i = 0; i < args.length; i++){
      if(i > 0) result += " " + operator + " ";
      result += args[i];
    }
    return result;
  }
  
  private String varList(String type, String varNames){
    // Splits a list of variables and inserts a type identifier between them.
    //System.out.println("  VarList: " + type + " + " + varNames);
    String[] vars = varNames.split(",");
    String result = "";
    for(int i = 0; i < vars.length; i++){
      if(i > 0) result += ", ";
      result += type + " " + vars[i].trim();
    }
    return result;
  }
  
//  public static ParseResult parse(Reader r){
//    STtoJava parser = new STtoJava(r);
//    if(parser.yyparse() != 0)
//		parser.addError(parser.getLine(), "Syntax error.", true);
//    return parser.getResult();
//  }
  
  /**
   * Parses a string of Structured Text source code and attempts to translate it into Java.
   * @param r
   */
  public ParseResult parse(Reader r){
  	lexer = new STLex(r, this);
  	yyparse();
    return getResult();
  }
  
  public String getTargetLanguage(){
  	return "Java";
  }
  
  public String getSourceLanguage(){
  	return "ST";
  }
  
  public static void main(String args[]) throws IOException {
  
    STtoJava yyparser;
    if ( args.length > 0 ) {
      // parse a file
      yyparser = new STtoJava();
	  ParseResult r = yyparser.parse(new FileReader(args[0]));
	  
	  System.out.println("\nSource:\n" + yyparser.getSource());
    }
    else {
      yyparser = new STtoJava();
	  ParseResult r = yyparser.parse(new FileReader(
			  "./org/oooneida/wb/translate/Test.st".replace('/', 
					  File.separatorChar)));
	  
	  if(r.hasErrors()){
	  	for(int i = 0; i < r.getErrors().size(); i++){
	  		System.err.println(r.getErrors().get(i));
	  	}
	  }
	  
	  System.out.println("\nSource:\n" + yyparser.getSource());
    }
  }
