//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA Workbench project under the Common Public License.
package org.oooneida.wb.tree;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.RowMapper;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;

import org.oooneida.wb.dom.events.ElementSelectionEvent;
import org.oooneida.wb.graph.popup.ContextMenu;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.events.MutationEvent;

/**
 * A tree for navigating the DOM of an IEC 61499 LibraryElement.
 * 
 * @author JHC
 * @version 20051109/JHC - Replaces DOMTreeModel.
 */
public class DOMTree extends JTree implements TreeCellRenderer,
        TreeSelectionListener, org.w3c.dom.events.EventListener {

    /**
	 * 
	 */
	private static final long serialVersionUID = -6831240633532660344L;

	protected ElementSelectionEvent selevt = new ElementSelectionEvent();

    protected JButton renderer = new JButton();

    protected Document document;
    
    protected int mouseButton = -1;
    protected MouseEvent mouseEvent = null;

    public DOMTree(Document doc) {
	super(DOMTreeNode.treeNodeForElement(doc.getDocumentElement()));
        document = doc;
        setCellRenderer(this);
        addTreeSelectionListener(this);
        renderer.setFont(new Font("Dialog", Font.BOLD, 12));
        renderer.setVerticalTextPosition(SwingConstants.CENTER);
        renderer.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        ((org.w3c.dom.events.EventTarget) doc).addEventListener(
                "MutationEvent", this, false);
                
        this.addMouseListener(new MouseAdapter() {
        	public void mousePressed(MouseEvent evt){
        		mouseButton = evt.getButton();
        		mouseEvent = evt;

    			TreePath clickedPath = getPathForLocation(evt.getX(), evt.getY());
        		if(mouseButton == MouseEvent.BUTTON3 ){
            		DOMTreeNode clickedNode = null;
            		if(clickedPath != null)
            			clickedNode = (DOMTreeNode)clickedPath.getLastPathComponent();
            		
            		TreePath selectedPath = getSelectionPath();
            		if(selectedPath != null){
            		DOMTreeNode selectedNode = (DOMTreeNode)selectedPath
        				.getLastPathComponent();
	        			if(selectedNode.equals(clickedNode)){
		        			// The node that was clicked on was already selected, so
		        			// create the menu now.
		                    Element el = selectedNode.getElement();
		                    if(el.getNodeName().equals("AlgorithmList")){
		                    	ContextMenu menu = new ContextMenu(el, mouseEvent);
		                		menu.create("Algorithm");
		                	}
		                	mouseEvent = null;
		                	mouseButton = -1;
	        			}
            		}
        		}
        		
        		setSelectionPath(clickedPath);
        	}
        });
    }

    /**
     * Issues an ElementSelectionEvent with HYPERACTIVATE detail (so GraphView knows to
     * draw a new diagram).
     */
    public void valueChanged(TreeSelectionEvent e) {
        DOMTreeNode selectedNode = (DOMTreeNode) e.getPath()
                .getLastPathComponent();
        Element el = selectedNode.getElement();
        
        java.lang.System.out.println("Button: " + mouseButton + " " + el.getNodeName());
        
        //Application needs to be handled differently.
        if(el.getNodeName().equals("Application"))
            el = (Element)el.getFirstChild();
        
        selevt.initSelectionEvent(el, this, ElementSelectionEvent.HYPERACTIVATE);
        ((org.w3c.dom.events.EventTarget) document).dispatchEvent(selevt);
        
        if(mouseButton == MouseEvent.BUTTON3 && mouseEvent != null){
        	if(el.getNodeName().equals("AlgorithmList")){
            	ContextMenu menu = new ContextMenu(el, mouseEvent);
        		menu.create("Algorithm");
        	}
        	mouseEvent = null;
        	mouseButton = -1;
        }
    }

    public void handleEvent(org.w3c.dom.events.Event evt) {
        if (!(evt instanceof MutationEvent))
            return;
        MutationEvent mutev = (MutationEvent) evt;
        Node munode = mutev.getRelatedNode();
        if (!(munode instanceof Element))
            return;
        if (DOMTreeNode.treeNodeForElement((Element) munode) == null)
            return;
        int attrChange = mutev.getAttrChange();
        if ((attrChange == MutationEvent.ADDITION) || (attrChange == MutationEvent.REMOVAL)) {
            //TODO process structure changes
        } else if (attrChange == MutationEvent.MODIFICATION) {
            treeDidChange();
        }
    }

    Color selectedColor = new Color(119, 190, 255);
    public void setSelectedColor(Color newColor){
	selectedColor = newColor;
    }
    public void setRendererEnabled(boolean isEnabled){
	renderer.setEnabled(isEnabled);
    }
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row,
            boolean hasFocus) {
        DOMTreeNode v = (DOMTreeNode) value;
        renderer.setText(v.getText());
        renderer.setIcon(v.getIcon());
        renderer.setForeground(Color.black);
        renderer.setBackground(selected ? selectedColor : Color.white);
        return renderer;
    }
}
