//Copyright (c)2005 Holobloc Inc.
//Contributed to the OOONEIDA FBench project under the Common Public License.
package org.oooneida.wb.tree;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.tree.TreeNode;

import org.oooneida.wb.IconFactory;
import org.oooneida.wb.dom.ElementModel;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A TreeNode encapsulating an Element of the DOM tree of an IEC 61499 XML
 * document that has been "scrubbed" of all but Element nodes.
 * 
 * @author JHC
 * @version 20051027/JHC - Redefined default icon.
 * @version 20050801/JHC - Modified from DOMTreeElement.
 * @version 20050419/JHC
 */
public class DOMTreeNode extends ElementModel implements TreeNode {
	
    /** Returns the count of displayable children. */
    public int getChildCount() {
        return getChildren().size();
    }

    /**
     * Returns the string to be used to display the tree node. Default is the
     * result of the ElementModel getName() method.
     */
    public String getText() {
        return getName();
    }

    /**
     * Returns the icon to be used for display of this element. Default is the
     * icon identified by the element's tag name in lower case, or <B>null </B>
     * if no icon is to be used.
     */
    public Icon getIcon() {
        return IconFactory.getIcon(element.getTagName().toLowerCase());
    }

    /**
     * Returns <b>true </b> if the given string is the name of an IEC 61499
     * LibraryElement with an interface.
     */
    public static boolean hasInterface(String nname) {
        return nname.endsWith("Type");
    }

    public TreeNode getChildAt(int childIndex) {
        Object ans = getChildren().get(childIndex);
        return (ans instanceof TreeNode) ? (TreeNode) ans : null;
    }

    public TreeNode getParent() {
        Node parent = getElement().getParentNode();
        return (parent instanceof Element) ? DOMTreeNode
                .treeNodeForElement((Element) parent) : null;
    }

    public int getIndex(TreeNode node) {
        return getChildren().indexOf(node);
    }

    /**
     * Returns <TT> true </TT> if the receiver allows children. Default return
     * value is <TT> false </TT> .
     */
    public boolean getAllowsChildren() {
        return false;
    }

    public boolean isLeaf() {
        return getChildren().isEmpty();
    }

    public Enumeration children() {
        return getChildren().elements();
    }

    /**
     * Returns a DOMTreeNode encapsulating the given DOM Element, or <TT>null
     * </TT> if no such DOMTreeNode exists.
     */
    public static DOMTreeNode treeNodeForElement(Element el) {
        ElementModel em = forElement(el, "org.oooneida.wb.tree.");
        if (em instanceof DOMTreeNode)
            return (DOMTreeNode) em;
        return null;
    }

    /**
     * Returns a Vector of the receiver's children. Default is a Vector of
     * TreeNodes encapsulating the Elements in the DOM element's list of
     * children.
     */
    public Vector getChildren() {
        Vector ans = new Vector();
        if (!getAllowsChildren())
            return ans;
        NodeList children = getElement().getChildNodes();
        
        //Node for containing list of VersionInfo
        Vector<DOMTreeNode> versionList = new Vector<DOMTreeNode>();
        boolean isVersionInfoAdded = false;
        for (int i = 0; i < children.getLength(); i++) {
        	DOMTreeNode childNode = DOMTreeNode
                    .treeNodeForElement((Element) children.item(i));
        	try{
            if(childNode.getElement().getNodeName().equals("VersionInfo")){
            	if(!isVersionInfoAdded){
            		Element verInfoElem = 
            			getElement().getOwnerDocument().createElement("VersionList");
                	DOMTreeNode versionListNode = treeNodeForElement(verInfoElem);
                	((VersionList) versionListNode).setVersionList(versionList);
                	ans.add(versionListNode);
                	isVersionInfoAdded = true;
            	}
            	versionList.add(childNode);
            }
            else if (childNode != null)
                ans.addElement(childNode);
        	}
        	catch(NullPointerException e){}
        }
        return ans;
    }
}