//Copyright (c)2005 Holobloc Inc., http://www.holobloc.com
//Licensed under the Academic Free License version 2.1
//See http://www.opensource.org/licenses/afl-2.1.php
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

/**
 * Checks all IEC 61499 XML files in the "src" subdirectory tree of the user
 * directory.
 * 
 * @author JHC
 * @version 20050518/JHC
 */
public class XMLChecker extends JFrame implements ErrorHandler, EntityResolver,
		ActionListener {

	private JTextArea ta = new JTextArea();

	private XMLReader reader;

	private JButton restartButton = new JButton("Restart");

	private String path = "";

	public XMLChecker() throws HeadlessException {
		super();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		initComponents();
		setVisible(true);

		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		try {
			reader = factory.newSAXParser().getXMLReader();
			reader.setErrorHandler(this);
			reader.setEntityResolver(this);
			checkAll();
		} catch (SAXParseException spe) {
			emit(spe, JOptionPane.ERROR_MESSAGE);
		} catch (SAXException sxe) {
			ta.append('\n' + sxe.toString() + '\n');
			Exception x = sxe.getException();
			if (x != null)
				ta.append(x.toString() + '\n');
		} catch (ParserConfigurationException pce) {
			ta.append('\n' + pce.toString() + '\n');
		}
	}

	private void initComponents() {
		setTitle("IEC 61499 XML Checker");
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridy = 1;
		this.getContentPane().add(restartButton, gbc);
		restartButton.addActionListener(this);
		//ta.setFont(new Font("Monospaced",10,Font.PLAIN));
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.fill = GridBagConstraints.BOTH;
		this.getContentPane().add(new JScrollPane(ta), gbc);
		setBounds(100, 100, 400, 400);
	}

	public void check(File fl) throws IOException, SAXException,
			SAXParseException {
		path = fl.getCanonicalPath();
		ta.append(path);
		ta.append("\n");
		if (fl.isDirectory()) {
			File[] files = fl.listFiles();
			for (int i = 0; i < files.length; i++)
				check((File) files[i]);
		} else
			reader.parse(new InputSource(new FileInputStream(fl)));
	}

	public static void main(String[] args) {
		new XMLChecker();
	}

	public void fatalError(SAXParseException exception) throws SAXException {
	}

	public void error(SAXParseException e) throws SAXParseException {
		throw e;
	}

	public void warning(SAXParseException err) throws SAXParseException {
		emit(err, JOptionPane.WARNING_MESSAGE);
	}

	public InputSource resolveEntity(String publicId, String systemId)
			throws SAXException, IOException {
		int n = systemId.lastIndexOf('/');
		if ((n >= 0) && (n < (systemId.length() - 1)))
			systemId = systemId.substring(n + 1);
		InputStream strm = getClass().getResourceAsStream(systemId);
		if (strm == null)
			throw new SAXException("Invalid System ID: " + systemId);
		else
			return new InputSource(strm);
	}

	private void emit(SAXParseException err, int type) {
		String[] mess = new String[3];
		mess[0] = "** line " + err.getLineNumber() + " column "
				+ err.getColumnNumber() + "***";
		mess[1] = err.getMessage();
		Exception x = err.getException();
		mess[2] = (x == null) ? "" : x.toString();
		JOptionPane.showMessageDialog(this, mess, path + " - Problem", type);
	}

	private void checkAll() {
		ta.setText("");
		try {
			check(new File("src"));
			String[] mess = {"All files have been checked.","No errors were found."};
			JOptionPane.showMessageDialog(this,mess,"XMLChecker - Finished",JOptionPane.INFORMATION_MESSAGE);
		} catch (SAXParseException spe) {
			emit(spe, JOptionPane.ERROR_MESSAGE);
		} catch (IOException ioe) {
			ta.append('\n' + ioe.toString() + '\n');
		} catch (SAXException sxe) {
			ta.append('\n' + sxe.toString() + '\n');
			Exception x = sxe.getException();
			if (x != null)
				ta.append(x.toString() + '\n');
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == restartButton)
			checkAll();
	}
}
